@@ -2,7 +2,7 @@
  *
  * redland.h - Redland RDF Application Framework public API
  *
- * $Id: librdf.h 11909 2007-02-17 06:58:52Z dajobe $
+ * $Id: librdf.h 13160 2007-11-06 11:45:18Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -34,18 +34,24 @@ extern "C" {
 
 #include <stdio.h>
 
-#ifdef WIN32
-#  ifdef REDLAND_STATIC
-#    define REDLAND_API
-#  else
-#    ifdef LIBRDF_INTERNAL
-#      define REDLAND_API _declspec(dllexport)
-#    else
-#      define REDLAND_API _declspec(dllimport)
+#ifndef REDLAND_API
+#  ifdef WIN32
+#    ifdef __GNUC__
+#      undef _declspec
+#      define _declspec(x) __declspec(x)
 #    endif
+#    ifdef REDLAND_STATIC
+#      define REDLAND_API
+#    else
+#      ifdef LIBRDF_INTERNAL
+#        define REDLAND_API _declspec(dllexport)
+#      else
+#        define REDLAND_API _declspec(dllimport)
+#      endif
+#    endif
+#  else
+#    define REDLAND_API
 #  endif
-#else
-#  define REDLAND_API
 #endif
 
 /* Use gcc 3.1+ feature to allow marking of deprecated API calls.
