@@ -2,7 +2,7 @@
  *
  * rdf_hash.h - RDF Hash Factory and Hash interfaces and definitions
  *
- * $Id: rdf_hash.h 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_hash.h 12192 2007-05-02 05:37:13Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -77,7 +77,10 @@ void librdf_hash_print(librdf_hash* hash, FILE *fh);
 REDLAND_API
 void librdf_hash_print_keys(librdf_hash* hash, FILE *fh);
 REDLAND_API
-void librdf_hash_print_values(librdf_hash* hash, char *key_string, FILE *fh);
+void librdf_hash_print_values(librdf_hash* hash, const char *key_string, FILE *fh);
+
+REDLAND_API
+unsigned char* librdf_hash_interpret_template(const unsigned char* template_string, librdf_hash* dictionary, const unsigned char* prefix,  const unsigned char* suffix);
 
 #ifdef __cplusplus
 }
