@@ -2,7 +2,7 @@
  *
  * rdf_hash_memory.c - RDF Hash In Memory Implementation
  *
- * $Id: rdf_hash_memory.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_hash_memory.c 12583 2007-09-21 11:19:32Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -116,7 +116,7 @@ static void librdf_hash_memory_cursor_finish(void* context);
 
 static int librdf_hash_memory_create(librdf_hash* new_hash, void* context);
 static int librdf_hash_memory_destroy(void* context);
-static int librdf_hash_memory_open(void* context, char *identifier, int mode, int is_writable, int is_new, librdf_hash* options);
+static int librdf_hash_memory_open(void* context, const char *identifier, int mode, int is_writable, int is_new, librdf_hash* options);
 static int librdf_hash_memory_close(void* context);
 static int librdf_hash_memory_clone(librdf_hash* new_hash, void *new_context, char *new_identifier, void* old_context);
 static int librdf_hash_memory_values_count(void *context);
@@ -350,13 +350,13 @@ librdf_hash_memory_destroy(void* context)
       
       /* this entry is used */
       if(node) {
-	librdf_hash_memory_node *next;
-	/* free all attached nodes */
-	while(node) {
-	  next=node->next;
-	  librdf_free_hash_memory_node(node);
-	  node=next;
-	}
+        librdf_hash_memory_node *next;
+        /* free all attached nodes */
+        while(node) {
+          next=node->next;
+          librdf_free_hash_memory_node(node);
+          node=next;
+        }
       }
     }
     LIBRDF_FREE(librdf_hash_memory_nodes, hcontext->nodes);
@@ -380,7 +380,7 @@ librdf_hash_memory_destroy(void* context)
  * Return value: non 0 on failure
  **/
 static int
-librdf_hash_memory_open(void* context, char *identifier,
+librdf_hash_memory_open(void* context, const char *identifier,
                         int mode, int is_writable, int is_new,
                         librdf_hash* options) 
 {
