@@ -2,9 +2,7 @@
  *
  * rdf_init.c - Redland library initialisation / termination
  *
- * $Id: rdf_init.c 11933 2007-02-25 20:02:09Z dajobe $
- *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -64,11 +62,15 @@
 #include <unistd.h>
 #endif
 
+#ifdef MODULAR_LIBRDF
+#include <ltdl.h>
+#endif
 
+#ifndef STANDALONE
 
-const char * const librdf_short_copyright_string = "Copyright 2000-2006 David Beckett. Copyright 2000-2005 University of Bristol";
+const char * const librdf_short_copyright_string = "Copyright 2000-2007 David Beckett. Copyright 2000-2005 University of Bristol";
 
-const char * const librdf_copyright_string = "Copyright (C) 2000-2006 David Beckett - http://purl.org/net/dajobe/\nCopyright (C) 2000-2005 University of Bristol - http://www.bristol.ac.uk/";
+const char * const librdf_copyright_string = "Copyright (C) 2000-2007 David Beckett - http://purl.org/net/dajobe/\nCopyright (C) 2000-2005 University of Bristol - http://www.bristol.ac.uk/";
 
 /**
  * librdf_version_string:
@@ -141,6 +143,9 @@ librdf_new_world(void) {
 
   world=(librdf_world*)LIBRDF_CALLOC(librdf_world, sizeof(librdf_world), 1);
 
+  if(!world)
+    return NULL;
+
 #ifdef HAVE_GETTIMEOFDAY
   if(!gettimeofday(&tv, &tz)) {
     world->genid_base=tv.tv_sec;
@@ -151,13 +156,21 @@ librdf_new_world(void) {
 #endif
   world->genid_counter=1;
   
+#ifdef MODULAR_LIBRDF
+  world->ltdl_opened = !(lt_dlinit());
+  if (world->ltdl_opened)
+    lt_dlsetsearchpath(LIBRDF_MODULE_DIR);
+  else
+    LIBRDF_DEBUG1("lt_dlinit() failed\n");
+#endif
+
   return world;
   
 }
 
 
 /**
- * librdf_free_world
+ * librdf_free_world:
  * @world: redland world object
  *
  * Terminate the library and frees all allocated resources.
@@ -204,6 +217,11 @@ librdf_free_world(librdf_world *world)
    }
 #endif
 
+#ifdef MODULAR_LIBRDF
+  if (world->ltdl_opened)
+    lt_dlexit();
+#endif
+
   LIBRDF_FREE(librdf_world, world);
 }
 
@@ -232,7 +250,7 @@ librdf_world_init_mutex(librdf_world* world)
 
 
 /**
- * librdf_world_open
+ * librdf_world_open:
  * @world: redland world object
  *
  * Open a created redland world environment.
@@ -486,7 +504,9 @@ librdf_world_get_genid(librdf_world* world)
 /* OLD INTERFACES BELOW HERE */
 
 /* For old interfaces below ONLY */
+#ifndef NO_STATIC_DATA
 static librdf_world* RDF_World;
+#endif
 
 /**
  * librdf_init_world:
@@ -503,12 +523,17 @@ static librdf_world* RDF_World;
 void
 librdf_init_world(char *digest_factory_name, void* not_used2)
 {
+#ifndef NO_STATIC_DATA
   RDF_World=librdf_new_world();
   if(!RDF_World)
     return;
   if(digest_factory_name)
     librdf_world_set_digest(RDF_World, digest_factory_name);
   librdf_world_open(RDF_World);
+#else
+  /* fail if NO_STATIC_DATA is defined */
+  abort();
+#endif
 }
 
 
@@ -525,7 +550,12 @@ librdf_init_world(char *digest_factory_name, void* not_used2)
 void
 librdf_destroy_world(void)
 {
+#ifndef NO_STATIC_DATA
   librdf_free_world(RDF_World);
+#else
+  /* fail if NO_STATIC_DATA is defined */
+  abort();
+#endif
 }
 
 
@@ -637,3 +667,30 @@ librdf_system_free(void *ptr)
 
 #endif
 
+
+#endif
+
+
+/* TEST CODE */
+
+
+#ifdef STANDALONE
+
+/* one more prototype */
+int main(int argc, char *argv[]);
+
+
+int
+main(int argc, char *argv[]) 
+{
+  librdf_world *world;
+
+  /* Minimal setup-cleanup test */
+  world=librdf_new_world();
+  librdf_free_world(world);
+
+  /* keep gcc -Wall happy */
+  return(0);
+}
+
+#endif
