@@ -2,7 +2,7 @@
  *
  * rdf_list.c - RDF List Implementation
  *
- * $Id: rdf_list.c 11930 2007-02-25 19:46:53Z dajobe $
+ * $Id: rdf_list.c 13287 2007-12-20 12:00:50Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -495,19 +495,21 @@ librdf_list_get_iterator(librdf_list* list)
   context->current=list->first;
   context->next=context->current != NULL ? context->current->next : NULL;
 
+  /* librdf_list_iterator_finished() calls librdf_list_remove_iterator_context(),
+   * librdf_list_iterator_finished() is called if librdf_new_iterator() fails */
+  librdf_list_add_iterator_context(list, context);
+
   iterator=librdf_new_iterator(list->world, 
                                (void*)context,
                                librdf_list_iterator_is_end,
                                librdf_list_iterator_next_method,
                                librdf_list_iterator_get_method,
                                librdf_list_iterator_finished);
-
-  if(iterator) {
+  if(!iterator)
+    librdf_list_iterator_finished(context);
+  else
     context->iterator=iterator;
-    librdf_list_add_iterator_context(list, context);
-  } else
-    librdf_list_iterator_finished(iterator);
-    
+
   return iterator;
 }
 
@@ -562,6 +564,9 @@ librdf_list_iterator_finished(void* iterator)
 {
   librdf_list_iterator_context* context=(librdf_list_iterator_context*)iterator;
 
+  if(!context)
+    return;
+  
   librdf_list_remove_iterator_context(context->list, context);
 
   LIBRDF_FREE(librdf_list_iterator_context, context);
