@@ -2,7 +2,7 @@
  *
  * rdf_model.h - RDF Model definition
  *
- * $Id: rdf_model.h 8508 2006-01-25 05:25:28Z cmdjb $
+ * $Id: rdf_model.h 11702 2006-12-05 04:16:55Z dajobe $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -43,11 +43,16 @@
 extern "C" {
 #endif
 
+/* class methods */
+
+REDLAND_API
+int librdf_model_enumerate(librdf_world* world, const unsigned int counter, const char **name, const char **label);
+
 /* constructors */
 
 /* Create a new Model */
 REDLAND_API
-librdf_model* librdf_new_model(librdf_world *world, librdf_storage *storage, char* options_string);
+librdf_model* librdf_new_model(librdf_world *world, librdf_storage *storage, const char* options_string);
 REDLAND_API
 librdf_model* librdf_new_model_with_options(librdf_world *world, librdf_storage *storage, librdf_hash* options);
 
@@ -68,9 +73,9 @@ int librdf_model_size(librdf_model* model);
 REDLAND_API
 int librdf_model_add(librdf_model* model, librdf_node* subject, librdf_node* predicate, librdf_node* object);
 REDLAND_API
-int librdf_model_add_string_literal_statement(librdf_model* model, librdf_node* subject, librdf_node* predicate, const unsigned char* literal, char *xml_language, int is_wf_xml);
+int librdf_model_add_string_literal_statement(librdf_model* model, librdf_node* subject, librdf_node* predicate, const unsigned char* literal, const char *xml_language, int is_wf_xml);
 REDLAND_API
-int librdf_model_add_typed_literal_statement(librdf_model* model, librdf_node* subject, librdf_node* predicate, const unsigned char* literal, char *xml_language, librdf_uri *datatype_uri);
+int librdf_model_add_typed_literal_statement(librdf_model* model, librdf_node* subject, librdf_node* predicate, const unsigned char* literal, const char *xml_language, librdf_uri *datatype_uri);
 REDLAND_API
 int librdf_model_add_statement(librdf_model* model, librdf_statement* statement);
 REDLAND_API
@@ -186,6 +191,18 @@ librdf_stream* librdf_model_find_statements_in_context(librdf_model* model, libr
 REDLAND_API
 librdf_iterator* librdf_model_get_contexts(librdf_model* model);
 
+REDLAND_API
+int librdf_model_transaction_start(librdf_model* model);
+REDLAND_API
+int librdf_model_transaction_start_with_handle(librdf_model* model, void* handle);
+REDLAND_API
+int librdf_model_transaction_commit(librdf_model* model);
+REDLAND_API
+int librdf_model_transaction_rollback(librdf_model* model);
+REDLAND_API
+void* librdf_model_transaction_get_handle(librdf_model* model);
+
+
 /**
  * LIBRDF_MODEL_FEATURE_CONTEXTS:
  *
