@@ -2,7 +2,7 @@
  *
  * rdf_query.c - RDF Query Results
  *
- * $Id: rdf_query_results.c 11930 2007-02-25 19:46:53Z dajobe $
+ * $Id: rdf_query_results.c 13061 2007-10-25 12:31:59Z laalto $
  *
  * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -281,36 +281,46 @@ librdf_query_results_to_counted_string(librdf_query_results *query_results,
                                        size_t *length_p)
 {
   librdf_query_results_formatter *formatter;
-  unsigned char *string=NULL;
+  void *string=NULL;
   size_t string_length=0;
   raptor_iostream *iostr;
+  int error=0;
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query_results, librdf_query_results, NULL);
 
-  iostr=raptor_new_iostream_to_string((void**)&string, &string_length, malloc);
+  iostr=raptor_new_iostream_to_string(&string, &string_length, malloc);
   if(!iostr)
     return NULL;
               
   formatter=librdf_new_query_results_formatter(query_results,
                                                NULL /* name */,
                                                format_uri);
-  if(!formatter)
+  if(!formatter) {
+    error=1;
     goto tidy;
+  }
 
   if(librdf_query_results_formatter_write(iostr, formatter,
                                           query_results, base_uri))
-    string=NULL;
+    error=1;
 
   librdf_free_query_results_formatter(formatter);
 
  tidy:
-  if(iostr)
-    raptor_free_iostream(iostr);
+  raptor_free_iostream(iostr);
 
-  if(length_p)
+  /* string is available only after the iostream is finished
+   * - clean it up here on error */
+  if(error) {
+    if(string) {
+      raptor_free_memory(string);
+      string=NULL;
+    }
+  }
+  else if(length_p)
     *length_p=string_length;
   
-  return string;
+  return (unsigned char *)string;
 }
 
 
@@ -627,7 +637,7 @@ void
 librdf_free_query_results_formatter(librdf_query_results_formatter* formatter) 
 {
   if(formatter->query_results->query->factory->free_results_formatter)
-    return formatter->query_results->query->factory->free_results_formatter(formatter);
+    formatter->query_results->query->factory->free_results_formatter(formatter);
 }
 
 
