@@ -2,9 +2,9 @@
  *
  * rdf_serializer.c - RDF Serializer (RDF triples to syntax) interface
  *
- * $Id: rdf_serializer.c 12151 2007-04-23 15:19:14Z dajobe $
+ * $Id: rdf_serializer.c 12992 2007-10-22 12:10:11Z laalto $
  *
- * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2002-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -71,9 +71,10 @@ librdf_init_serializer(librdf_world *world)
 void
 librdf_finish_serializer(librdf_world *world) 
 {
-  raptor_free_sequence(world->serializers);
-  world->serializers=NULL;
-
+  if(world->serializers) {
+    raptor_free_sequence(world->serializers);
+    world->serializers=NULL;
+  }
 #ifdef HAVE_RAPTOR_RDF_SERIALIZER
   librdf_serializer_raptor_destructor();
 #endif
@@ -116,75 +117,67 @@ librdf_serializer_register_factory(librdf_world *world,
                                    void (*factory) (librdf_serializer_factory*))
 {
   librdf_serializer_factory *serializer;
-  char *name_copy;
-  char *label_copy;
 
   librdf_world_open(world);
 
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG2("Received registration for serializer %s\n", name);
 #endif
-  
-  if(!world->serializers)
+
+  if(!world->serializers) {
     world->serializers=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_serializer_factory, NULL);
-  
+    if(!world->serializers)
+      goto oom;
+  }
+
   serializer=(librdf_serializer_factory*)LIBRDF_CALLOC(librdf_serializer_factory, 1,
                                                        sizeof(librdf_serializer_factory));
   if(!serializer)
-    LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
-  
-  name_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(name)+1);
-  if(!name_copy) {
-    librdf_free_serializer_factory(serializer);
-    LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
-  }
-  strcpy(name_copy, name);
-  serializer->name=name_copy;
+    goto oom;
+
+  serializer->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  if(!serializer->name)
+    goto oom_tidy;
+  strcpy(serializer->name, name);
 
   if(label) {
-    label_copy=(char*)LIBRDF_CALLOC(cstring, strlen(label)+1, 1);
-    if(!label_copy) {
-      librdf_free_serializer_factory(serializer);
-      LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
-    }
-    strcpy(label_copy, label);
-    serializer->label=label_copy;
+    serializer->label=(char*)LIBRDF_MALLOC(cstring, strlen(label)+1);
+    if(!serializer->label)
+      goto oom_tidy;
+    strcpy(serializer->label, label);
   }
-        
+
   /* register mime type if any */
   if(mime_type) {
-    char *mime_type_copy;
-    mime_type_copy=(char*)LIBRDF_CALLOC(cstring, 1, strlen(mime_type)+1);
-    if(!mime_type_copy) {
-      librdf_free_serializer_factory(serializer);
-      LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
-    }
-    strcpy(mime_type_copy, mime_type);
-    serializer->mime_type=mime_type_copy;
+    serializer->mime_type=(char*)LIBRDF_MALLOC(cstring, strlen(mime_type)+1);
+    if(!serializer->mime_type)
+      goto oom_tidy;
+    strcpy(serializer->mime_type, mime_type);
   }
 
   /* register URI if any */
   if(uri_string) {
-    librdf_uri *uri;
-
-    uri=librdf_new_uri(world, uri_string);
-    if(!uri) {
-      librdf_free_serializer_factory(serializer);
-      LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
-    }
-    serializer->type_uri=uri;
+    serializer->type_uri=librdf_new_uri(world, uri_string);
+    if(!serializer->type_uri)
+      goto oom_tidy;
   }
-  
-        
+
+  if(raptor_sequence_push(world->serializers, serializer)) 
+    goto oom;
+
   /* Call the serializer registration function on the new object */
   (*factory)(serializer);
 
-
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG3("%s has context size %d\n", name, serializer->context_length);
 #endif
-  
-  raptor_sequence_push(world->serializers, serializer);
+
+  return;
+
+  oom_tidy:
+  librdf_free_serializer_factory(serializer);
+  oom:
+  LIBRDF_FATAL1(world, LIBRDF_FROM_SERIALIZER, "Out of memory");
 }
 
 
@@ -361,7 +354,10 @@ librdf_new_serializer_from_factory(librdf_world *world,
   d->factory=factory;
 
   if(factory->init)
-    factory->init(d, d->context);
+    if(factory->init(d, d->context)) {
+      librdf_free_serializer(d);
+      return NULL;
+    }
 
   return d;
 }
@@ -793,13 +789,11 @@ librdf_serializer_set_feature(librdf_serializer* serializer,
 /**
  * librdf_serializer_set_namespace:
  * @serializer: serializer object
- * @uri: URI of namespace
- * @prefix: prefix to use
+ * @uri: URI of namespace or NULL
+ * @prefix: prefix to use or NULL
  *
  * Set a namespace URI/prefix mapping.
  * 
- * NOTE: May not be implemented yet - depends on raptor_serializer work.
- * 
  * Return value: non 0 on failure
  **/
   
@@ -808,8 +802,10 @@ librdf_serializer_set_namespace(librdf_serializer* serializer,
                                 librdf_uri *uri, const char *prefix) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(serializer, librdf_serializer, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, librdf_uri, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(prefix, string, 1);
+  if(uri && !*librdf_uri_as_string(uri))
+    uri=NULL;
+  if(prefix && !*prefix)
+    prefix=NULL;
 
   if(serializer->factory->set_namespace)
     return serializer->factory->set_namespace(serializer->context, uri, prefix);
