@@ -2,7 +2,7 @@
  *
  * rdf_serializer_raptor.c - RDF Serializers via Raptor
  *
- * $Id: rdf_serializer_raptor.c 11930 2007-02-25 19:46:53Z dajobe $
+ * $Id: rdf_serializer_raptor.c 13149 2007-11-06 08:28:33Z laalto $
  *
  * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -103,7 +103,7 @@ librdf_serializer_raptor_terminate(void *context)
 static librdf_node*
 librdf_serializer_raptor_get_feature(void *context, librdf_uri* feature) {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
-  static unsigned char intbuffer[20]; /* FIXME */
+  unsigned char intbuffer[20]; /* FIXME */
   unsigned char *uri_string;
   raptor_feature feature_i;
   
@@ -187,7 +187,7 @@ librdf_serializer_raptor_serialize_statement(raptor_serializer *rserializer,
   if(!librdf_node_is_resource(predicate)) {
     librdf_log(statement->world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
-               "Do not know how to print triple predicate type %d",
+               "Do not know how to serialize triple predicate type %d",
                librdf_node_get_type(predicate));
     return 1;
   }
@@ -220,7 +220,7 @@ librdf_serializer_raptor_serialize_statement(raptor_serializer *rserializer,
     default:
       librdf_log(statement->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
-                 "Do not know how to print triple object type %d",
+                 "Do not know how to serialize triple object type %d",
                  librdf_node_get_type(object));
       return 1;
   }
@@ -266,8 +266,11 @@ librdf_serializer_raptor_serialize_stream_to_file_handle(void *context,
   /* start the serialize */
   rc=raptor_serialize_start_to_file_handle(scontext->rdf_serializer, 
                                            (raptor_uri*)base_uri, handle);
-  if(rc)
+  if(rc) {
+    /* free up resources on error */
+    raptor_serialize_end(scontext->rdf_serializer);
     return 1;
+  }
 
   scontext->errors=0;
   scontext->warnings=0;
@@ -275,16 +278,19 @@ librdf_serializer_raptor_serialize_stream_to_file_handle(void *context,
                                       librdf_serializer_raptor_error_handler);
   raptor_serializer_set_warning_handler(scontext->rdf_serializer, scontext, 
                                         librdf_serializer_raptor_warning_handler);
-    
+
+  rc=0;
   while(!librdf_stream_end(stream)) {
     librdf_statement *statement=librdf_stream_get_object(stream);
-    librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
-                                                 statement);
+    rc=librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
+                                                    statement);
+    if(rc)
+      break;
     librdf_stream_next(stream);
   }
   raptor_serialize_end(scontext->rdf_serializer);
 
-  return 0;
+  return rc;
 }
 
 
@@ -318,7 +324,7 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
 {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
   raptor_iostream *iostr;
-  unsigned char *string=NULL;
+  void *string=NULL;
   size_t string_length=0;
   int rc=0;
 
@@ -326,7 +332,7 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
     return NULL;
 
   /* start the serialize */
-  iostr=raptor_new_iostream_to_string((void**)&string, &string_length,
+  iostr=raptor_new_iostream_to_string(&string, &string_length,
                                       malloc);
   if(!iostr) {
     librdf_free_stream(stream);    
@@ -347,19 +353,27 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
                                       librdf_serializer_raptor_error_handler);
   raptor_serializer_set_warning_handler(scontext->rdf_serializer, scontext, 
                                         librdf_serializer_raptor_warning_handler);
-    
+
+  rc=0;    
   while(!librdf_stream_end(stream)) {
     librdf_statement *statement=librdf_stream_get_object(stream);
-    librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
-                                                 statement);
+    rc=librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
+                                                    statement);
+    if(rc)
+      break;
     librdf_stream_next(stream);
   }
   raptor_serialize_end(scontext->rdf_serializer);
 
+  if(rc) {
+    free(string);
+    return NULL;
+  }
+
   if(length_p)
     *length_p=string_length;
   
-  return string;
+  return (unsigned char *)string;
 }
 
 
@@ -416,16 +430,19 @@ librdf_serializer_raptor_serialize_stream_to_iostream(void *context,
                                       librdf_serializer_raptor_error_handler);
   raptor_serializer_set_warning_handler(scontext->rdf_serializer, scontext, 
                                         librdf_serializer_raptor_warning_handler);
-    
+
+  rc=0;
   while(!librdf_stream_end(stream)) {
     librdf_statement *statement=librdf_stream_get_object(stream);
-    librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
-                                                 statement);
+    rc=librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
+                                                    statement);
+    if(rc)
+      break;
     librdf_stream_next(stream);
   }
   raptor_serialize_end(scontext->rdf_serializer);
 
-  return 0;
+  return rc;
 }
 
 
