@@ -2,7 +2,7 @@
  *
  * rdf_statement.c - RDF Triple (Statement) interface
  *
- * $Id: rdf_statement.c 12180 2007-05-01 03:42:32Z dajobe $
+ * $Id: rdf_statement.c 13148 2007-11-06 08:26:25Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
@@ -442,7 +442,7 @@ librdf_statement_to_string(librdf_statement *statement)
   int statement_string_len=0;
   const char *format;
 #define NULL_STRING_LENGTH 6
-  static const unsigned char *null_string=(const unsigned char *)"(null)";
+  static const unsigned char * const null_string=(const unsigned char *)"(null)";
   size_t len;
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, NULL);
