@@ -2,7 +2,7 @@
  *
  * rdf_storage_hashes.c - RDF Storage as Hashes Implementation
  *
- * $Id: rdf_storage_hashes.c 11930 2007-02-25 19:46:53Z dajobe $
+ * $Id: rdf_storage_hashes.c 13287 2007-12-20 12:00:50Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -243,6 +243,8 @@ librdf_storage_hashes_init_common(librdf_storage* storage, const char *name,
   int index_contexts=0;
   int hash_count=0;
   
+  context->name=(char*)name;
+  
   context->hash_type=hash_type;
   context->db_dir=db_dir;
   context->indexes=indexes;
@@ -271,12 +273,17 @@ librdf_storage_hashes_init_common(librdf_storage* storage, const char *name,
 
   /* Start allocating the arrays */
   context->hashes=(librdf_hash**)LIBRDF_CALLOC(librdf_hash, hash_count, sizeof(librdf_hash*));
-  if(!context->hashes)
+  if(!context->hashes) {
+    if(context->name)
+      LIBRDF_FREE(cstring, context->name);
     return 1;
+  }
 
   context->hash_descriptions=(librdf_hash_descriptor**)LIBRDF_CALLOC(librdf_hash_descriptor, hash_count, sizeof(librdf_hash_descriptor*));
   if(!context->hash_descriptions) {
     LIBRDF_FREE(librdf_hash, context->hashes);
+    if(context->name)
+      LIBRDF_FREE(cstring, context->name);
     return 1;
   }
   
@@ -284,6 +291,8 @@ librdf_storage_hashes_init_common(librdf_storage* storage, const char *name,
   if(!context->names) {
     LIBRDF_FREE(librdf_hash, context->hashes);
     LIBRDF_FREE(librdf_hash_descriptor, context->hash_descriptions);
+    if(context->name)
+      LIBRDF_FREE(cstring, context->name);
     return 1;
   }
   
@@ -350,6 +359,7 @@ librdf_storage_hashes_init(librdf_storage* storage, const char *name,
 {
   char *hash_type, *db_dir, *indexes;
   int mode, is_writable, is_new;
+  char *name_copy=NULL;
   
   if(!options)
     return 1;
@@ -370,9 +380,15 @@ librdf_storage_hashes_init(librdf_storage* storage, const char *name,
   
   if((is_new=librdf_hash_get_as_boolean(options, "new"))<0)
     is_new=0; /* default is NOT NEW */
-  
 
-  return librdf_storage_hashes_init_common(storage, name, 
+  if(name) {
+    name_copy = (char *)LIBRDF_MALLOC(cstring, strlen(name)+1);
+    if(!name_copy)
+      return 1;
+    strcpy(name_copy, name);
+  }
+
+  return librdf_storage_hashes_init_common(storage, name_copy, 
                                            hash_type, db_dir, indexes,
                                            mode, is_writable, is_new, 
                                            options);
@@ -421,6 +437,8 @@ librdf_storage_hashes_terminate(librdf_storage* storage)
   if(context->value_buffer)
     LIBRDF_FREE(data, context->value_buffer);
 
+  if(context->name)
+    LIBRDF_FREE(cstring, context->name);
 }
 
 
@@ -429,16 +447,19 @@ librdf_storage_hashes_clone(librdf_storage* new_storage, librdf_storage* old_sto
 {
   librdf_storage_hashes_context *old_context=(librdf_storage_hashes_context*)old_storage->context;
   librdf_storage_hashes_context *new_context=(librdf_storage_hashes_context*)new_storage->context;
-  librdf_hash *options;
-  
-  new_context->name=librdf_heuristic_gen_name(old_context->name);
-  if(!new_context->name)
-    return 1;
 
+  /* Bump up old context name if any */
+  if(old_context->name) {  
+    new_context->name=librdf_heuristic_gen_name(old_context->name);
+    if(!new_context->name)
+      return 1;
+  }
+  
   /* This is always a copy of an in-memory hash */
-  options=librdf_new_hash_from_hash(old_context->options);
-  if(!options) {
-    LIBRDF_FREE(cstring, new_context->name);
+  new_context->options=librdf_new_hash_from_hash(old_context->options);
+  if(!new_context->options) {
+    if(new_context->name)
+      LIBRDF_FREE(cstring, new_context->name);
     return 1;
   }
 
@@ -449,9 +470,10 @@ librdf_storage_hashes_clone(librdf_storage* new_storage, librdf_storage* old_sto
                                        old_context->mode,
                                        old_context->is_writable,
                                        old_context->is_new,
-                                       options)) {
-    librdf_free_hash(options);
-    LIBRDF_FREE(cstring, new_context->name);
+                                       new_context->options)) {
+    librdf_free_hash(new_context->options);
+    if(new_context->name)
+      LIBRDF_FREE(cstring, new_context->name);
     return 1;
   }
 
@@ -487,12 +509,6 @@ librdf_storage_hashes_open(librdf_storage* storage, librdf_model* model)
       break;
   }
 
-  /* on success or failure - don't need the passed in options any more */
-  if(context->options) {
-    librdf_free_hash(context->options);
-    context->options=NULL;
-  }
-  
   return result;
 }
 
@@ -1227,10 +1243,10 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
 {
   librdf_storage_hashes_context *scontext=(librdf_storage_hashes_context*)storage->context;
   librdf_storage_hashes_node_iterator_context* icontext;
-  librdf_iterator *iterator;
   librdf_hash *hash;
   librdf_statement_part fields;
   unsigned char *key_buffer;
+  librdf_iterator* iterator;
   
   icontext=(librdf_storage_hashes_node_iterator_context*)LIBRDF_CALLOC(librdf_storage_hashes_node_iterator_context, 1, sizeof(librdf_storage_hashes_node_iterator_context));
   if(!icontext)
@@ -1330,7 +1346,6 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
 
   LIBRDF_FREE(data, key_buffer);
 
-
   iterator=librdf_new_iterator(storage->world,
                                (void*)icontext,
                                librdf_storage_hashes_node_iterator_is_end,
@@ -1339,7 +1354,6 @@ librdf_storage_hashes_node_iterator_create(librdf_storage* storage,
                                librdf_storage_hashes_node_iterator_finished);
   if(!iterator)
     librdf_storage_hashes_node_iterator_finished(icontext);
-
   return iterator;
 }
 
@@ -1803,12 +1817,9 @@ librdf_storage_hashes_get_contexts(librdf_storage* storage)
                                &librdf_storage_hashes_get_contexts_next_method,
                                &librdf_storage_hashes_get_contexts_get_method,
                                &librdf_storage_hashes_get_contexts_finished);
-  if(!iterator) {
-    librdf_storage_hashes_get_contexts_finished((void*)icontext);
-    return NULL;
-  }
-  
-  return iterator;  
+  if(!iterator)
+    librdf_storage_hashes_get_contexts_finished(icontext);
+  return iterator;
 }
 
 
