@@ -2,7 +2,7 @@
  *
  * rdf_storage_internal.h - Internal RDF Storage definitions
  *
- * $Id: rdf_storage_internal.h 11726 2006-12-07 08:08:08Z dajobe $
+ * $Id: rdf_storage_internal.h 13134 2007-10-31 13:39:38Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -201,13 +201,6 @@ void librdf_finish_storage(librdf_world *world);
 /* class methods */
 librdf_storage_factory* librdf_get_storage_factory(librdf_world* world, const char *name);
 
-void librdf_storage_add_reference(librdf_storage *storage);
-void librdf_storage_remove_reference(librdf_storage *storage);
-
-#ifdef HAVE_SQLITE
-void librdf_init_storage_sqlite(librdf_world *world);
-#endif
-
 
 /* rdf_storage_sql.c */
 typedef struct  
