@@ -2,7 +2,7 @@
  *
  * rdf_storage_list.c - RDF Storage in memory as a list implementation
  *
- * $Id: rdf_storage_list.c 11930 2007-02-25 19:46:53Z dajobe $
+ * $Id: rdf_storage_list.c 13287 2007-12-20 12:00:50Z laalto $
  *
  * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -197,26 +197,16 @@ static int
 librdf_storage_list_close(librdf_storage* storage)
 {
   librdf_storage_list_context* context=(librdf_storage_list_context*)storage->context;
-  int status=0;
-  librdf_iterator* iterator;
   
   if(context->list) {
-    iterator=librdf_list_get_iterator(context->list);
-    status=(iterator != 0);
-    if(iterator) {
-      while(!librdf_iterator_end(iterator)) {
-        librdf_storage_list_node* sln=(librdf_storage_list_node*)librdf_iterator_get_object(iterator);
-        librdf_free_statement(sln->statement);
-        if(sln->context)
-          librdf_free_node(sln->context);
-        LIBRDF_FREE(librdf_storage_list_node, sln);
-
-        librdf_iterator_next(iterator);
-      }
-      librdf_free_iterator(iterator);
+    librdf_storage_list_node* sln;
+    while((sln=(librdf_storage_list_node*)librdf_list_pop(context->list))) {
+      librdf_free_statement(sln->statement);
+      if(sln->context)
+        librdf_free_node(sln->context);
+      LIBRDF_FREE(librdf_storage_list_node, sln);
     }
     librdf_free_list(context->list);
-
     context->list=NULL;
   }
 
@@ -227,7 +217,7 @@ librdf_storage_list_close(librdf_storage* storage)
     }
   }
   
-  return status;
+  return 0;
 }
 
 
@@ -349,8 +339,10 @@ librdf_storage_list_serialise(librdf_storage* storage)
 
   scontext->index_contexts=context->index_contexts;
   scontext->iterator=librdf_list_get_iterator(context->list);
-  if(!scontext->iterator)
+  if(!scontext->iterator) {
+    LIBRDF_FREE(librdf_storage_list_serialise_stream_context, scontext);
     return librdf_new_empty_stream(storage->world);
+  }
     
   
   scontext->storage=storage;
@@ -451,9 +443,18 @@ librdf_storage_list_find_statements(librdf_storage* storage, librdf_statement* s
     return NULL;
   
   stream=librdf_storage_list_serialise(storage);
-  if(stream)
-    librdf_stream_add_map(stream, &librdf_stream_statement_find_map,
-                          (librdf_stream_map_free_context_handler)&librdf_free_statement, (void*)statement);
+  if(stream) {
+    if(librdf_stream_add_map(stream, &librdf_stream_statement_find_map,
+                             (librdf_stream_map_free_context_handler)&librdf_free_statement,
+                             (void*)statement)) {
+      /* error - stream_add_map failed */
+      librdf_free_stream(stream);
+      stream=NULL;
+    }
+  }
+  else
+    librdf_free_statement(statement);
+
   return stream;
 }
 
@@ -894,9 +895,8 @@ librdf_storage_list_get_contexts(librdf_storage* storage)
                                &librdf_storage_list_get_contexts_get_method,
                                &librdf_storage_list_get_contexts_finished);
   if(!iterator)
-    librdf_storage_list_get_contexts_finished((void*)icontext);
-
-  return iterator;  
+    librdf_storage_list_get_contexts_finished(icontext);
+  return iterator;
 }
 
 
