@@ -2,12 +2,12 @@
  *
  * rdf_storage_mysql.c - RDF Storage in MySQL DB interface definition.
  *
- * $Id: rdf_storage_mysql.c 12201 2007-05-04 20:11:29Z dajobe $
+ * $Id: rdf_storage_mysql.c 13287 2007-12-20 12:00:50Z laalto $
  *
  * Based in part on rdf_storage_list and rdf_storage_parka.
  *
  * Copyright (C) 2003-2005 Morten Frederiksen - http://purl.org/net/morten/
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -2486,7 +2486,7 @@ LEFT JOIN Resources AS R ON S.Context=R.ID \
 LEFT JOIN Bnodes AS B ON S.Context=B.ID \
 LEFT JOIN Literals AS L ON S.Context=L.ID";
   char *query;
-  librdf_iterator *iterator;
+  librdf_iterator* iterator;
 
   /* Initialize get_contexts context */
   if(!(gccontext=(librdf_storage_mysql_get_contexts_context*)
@@ -2540,8 +2540,7 @@ LEFT JOIN Literals AS L ON S.Context=L.ID";
                                &librdf_storage_mysql_get_contexts_get_context,
                                &librdf_storage_mysql_get_contexts_finished);
   if(!iterator)
-    librdf_storage_mysql_get_contexts_finished((void*)gccontext);
-
+    librdf_storage_mysql_get_contexts_finished(gccontext);
   return iterator;
 }
 
@@ -2804,6 +2803,7 @@ librdf_storage_mysql_transaction_commit(librdf_storage* storage)
 #ifdef LIBRDF_DEBUG_SQL
     LIBRDF_DEBUG1("Nothing pending to commit\n");
 #endif
+    librdf_storage_mysql_transaction_terminate(storage);
     return 0;
   }
 #ifdef LIBRDF_DEBUG_SQL
