@@ -2,7 +2,7 @@
  *
  * rdf_storage_sqlite.c - RDF Storage using SQLite implementation
  *
- * $Id: rdf_storage_sqlite.c 11930 2007-02-25 19:46:53Z dajobe $
+ * $Id: rdf_storage_sqlite.c 13296 2007-12-20 13:02:01Z laalto $
  *
  * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -80,7 +80,7 @@
 #endif
 
 
-static const char* sqlite_synchronous_flags[4] = {
+static const char* const sqlite_synchronous_flags[4] = {
   "off", "normal", "full", NULL
 };
 
@@ -108,6 +108,7 @@ typedef struct
   int in_stream;
   librdf_storage_sqlite_query *in_stream_queries;
 
+  int in_transaction;
 } librdf_storage_sqlite_context;
 
 
@@ -157,6 +158,11 @@ static int librdf_storage_sqlite_get_contexts_next_method(void* iterator);
 static void* librdf_storage_sqlite_get_contexts_get_method(void* iterator, int);
 static void librdf_storage_sqlite_get_contexts_finished(void* iterator);
 
+/* transactions */
+static int librdf_storage_sqlite_transaction_start(librdf_storage *storage);
+static int librdf_storage_sqlite_transaction_commit(librdf_storage *storage);
+static int librdf_storage_sqlite_transaction_rollback(librdf_storage *storage);
+
 static void librdf_storage_sqlite_query_flush(librdf_storage *storage);
 
 static void librdf_storage_sqlite_register_factory(librdf_storage_factory *factory);
@@ -170,22 +176,25 @@ librdf_storage_sqlite_init(librdf_storage* storage, const char *name,
 {
   librdf_storage_sqlite_context *context=(librdf_storage_sqlite_context*)storage->context;
   char *name_copy;
-  int is_new;
   char* synchronous;
   
-  if(!name)
+  if(!name) {
+    librdf_free_hash(options);
     return 1;
+  }
   
   context->storage=storage;
 
   context->name_len=strlen(name);
   name_copy=(char*)LIBRDF_MALLOC(cstring, context->name_len+1);
-  if(!name_copy)
+  if(!name_copy) {
+    librdf_free_hash(options);
     return 1;
+  }
   strncpy(name_copy, name, context->name_len+1);
   context->name=name_copy;
   
-  if((is_new=librdf_hash_get_as_boolean(options, "new"))>0)
+  if(librdf_hash_get_as_boolean(options, "new")>0)
     context->is_new=1; /* default is NOT NEW */
 
   /* Redland default is "PRAGMA synchronous normal" */
@@ -276,7 +285,7 @@ typedef enum {
   TRIPLE_NONE   =3,
 } triple_node_type;
 
-static const char * triples_fields[4][3] = {
+static const char * const triples_fields[4][3] = {
   { "subjectUri",   "subjectBlank", NULL },
   { "predicateUri", NULL,           NULL },
   { "objectUri",    "objectBlank",  "objectLiteral" },
@@ -291,7 +300,7 @@ librdf_storage_sqlite_get_1int_callback(void *arg,
   int* count_p=(int*)arg;
   
   if(argc == 1) {
-    *count_p=atoi(argv[0]);
+    *count_p=argv[0] ? atoi(argv[0]) : 0;
   }
   return 0;
 }
@@ -312,6 +321,8 @@ sqlite_string_escape(const unsigned char *raw, size_t raw_len, size_t *len_p)
 
   len= raw_len+escapes+2; /* for '' */
   escaped=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
+  if(!escaped)
+    return NULL;
 
   p=escaped;
   *p++='\'';
@@ -342,6 +353,10 @@ librdf_storage_sqlite_exec(librdf_storage* storage,
   int status=SQLITE_OK;
   char *errmsg=NULL;
 
+  /* sqlite crashes if passed in a NULL sql string */
+  if(!request)
+    return 1;
+
   LIBRDF_DEBUG2("SQLite exec '%s'\n", request);
   
   status=sqlite_EXEC(context->db, (const char*)request, callback, arg, &errmsg);
@@ -350,31 +365,44 @@ librdf_storage_sqlite_exec(librdf_storage* storage,
   
   if(status != SQLITE_OK) {
     if(status == SQLITE_LOCKED && !callback && context->in_stream) {
-       librdf_storage_sqlite_query *query;
-       query=(librdf_storage_sqlite_query*)LIBRDF_CALLOC(librdf_storage_sqlite_query, 1, sizeof(librdf_storage_sqlite_query));
+      librdf_storage_sqlite_query *query;
+      /* error message from sqlite_EXEC needs to be freed on both sqlite 2 and 3 */
+      if(errmsg)
+        sqlite_FREE(errmsg);
 
-       query->query=(unsigned char*)LIBRDF_MALLOC(cstring, strlen((char *)request)+1);
-       strcpy((char*)query->query, (char *)request);
 
-       if(!context->in_stream_queries)
-         context->in_stream_queries=query;
-       else {
-         librdf_storage_sqlite_query *q=context->in_stream_queries;
+      query=(librdf_storage_sqlite_query*)LIBRDF_CALLOC(librdf_storage_sqlite_query, 1, sizeof(librdf_storage_sqlite_query));
+      if(!query)
+        return 1;
 
-         while(q->next)
-           q=q->next;
+      query->query=(unsigned char*)LIBRDF_MALLOC(cstring, strlen((char *)request)+1);
+      if(!query->query) {
+        LIBRDF_FREE(librdf_storage_sqlite_query, query);
+        return 1;
+      }
 
-         q->next=query;
-       }
+      strcpy((char*)query->query, (char *)request);
 
-       status = SQLITE_OK;
+      if(!context->in_stream_queries)
+        context->in_stream_queries=query;
+      else {
+        librdf_storage_sqlite_query *q=context->in_stream_queries;
+
+        while(q->next)
+          q=q->next;
+
+        q->next=query;
+      }
+
+      status = SQLITE_OK;
 
     } else {
       librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "SQLite database %s SQL exec '%s' failed - %s (%d)",
                  context->name, request, errmsg, status);
-      sqlite_CLOSE(context->db);
-      context->db=NULL;
+      /* error message from sqlite_EXEC needs to be freed on both sqlite 2 and 3 */
+      if(errmsg)
+        sqlite_FREE(errmsg);
     }
   }
 
@@ -394,6 +422,9 @@ librdf_storage_sqlite_set_helper(librdf_storage *storage,
   unsigned char *request;
 
   sb=raptor_new_stringbuffer();
+  if(!sb)
+    return -1;
+
   raptor_stringbuffer_append_string(sb, 
                                     (const unsigned char*)"INSERT INTO ", 1);
   raptor_stringbuffer_append_string(sb,  
@@ -435,6 +466,9 @@ librdf_storage_sqlite_get_helper(librdf_storage *storage,
   unsigned char *request;
 
   sb=raptor_new_stringbuffer();
+  if(!sb)
+    return -1;
+
   raptor_stringbuffer_append_string(sb,  
                                     (const unsigned char*)"SELECT id FROM ", 1);
   raptor_stringbuffer_append_string(sb,  
@@ -471,15 +505,17 @@ librdf_storage_sqlite_uri_helper(librdf_storage* storage,
   unsigned char *expression=NULL;
   unsigned char *uri_e;
   size_t uri_e_len;
-  int id;
-  static const char *field="uri";
+  int id=-1;
+  static const char * const field="uri";
 
   uri_string=librdf_uri_as_counted_string(uri, &uri_len);
   uri_e=sqlite_string_escape(uri_string, uri_len, &uri_e_len);
   if(!uri_e)
-    return -1;
+    goto tidy;
   
   expression=(unsigned char*)LIBRDF_MALLOC(cstring, strlen(field)+3+uri_e_len+1);
+  if(!expression)
+    goto tidy;
   sprintf((char*)expression, "%s = %s", field, uri_e);
   id=librdf_storage_sqlite_get_helper(storage, TABLE_URIS, expression);
   if(id >=0)
@@ -490,7 +526,8 @@ librdf_storage_sqlite_uri_helper(librdf_storage* storage,
   tidy:
   if(expression)
     LIBRDF_FREE(cstring, expression);
-  LIBRDF_FREE(cstring, uri_e);
+  if(uri_e)
+    LIBRDF_FREE(cstring, uri_e);
 
   return id;
 }
@@ -504,15 +541,17 @@ librdf_storage_sqlite_blank_helper(librdf_storage* storage,
   unsigned char *expression=NULL;
   unsigned char *blank_e;
   size_t blank_e_len;
-  int id;
-  static const char *field="blank";
+  int id=-1;
+  static const char * const field="blank";
 
   blank_len=strlen((const char*)blank);
   blank_e=sqlite_string_escape(blank, blank_len, &blank_e_len);
   if(!blank_e)
-    return -1;
+    goto tidy;
   
   expression=(unsigned char*)LIBRDF_MALLOC(cstring, strlen(field)+3+blank_e_len+1);
+  if(!expression)
+    goto tidy;
   sprintf((char*)expression, "%s = %s", field, blank_e);
   id=librdf_storage_sqlite_get_helper(storage, TABLE_BLANKS, expression);
   if(id >=0)
@@ -523,7 +562,8 @@ librdf_storage_sqlite_blank_helper(librdf_storage* storage,
   tidy:
   if(expression)
     LIBRDF_FREE(cstring, expression);
-  LIBRDF_FREE(cstring, blank_e);
+  if(blank_e)
+    LIBRDF_FREE(cstring, blank_e);
 
   return id;
 }
@@ -536,21 +576,23 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
                                      const char *language,
                                      librdf_uri *datatype) 
 {
-  int id;
+  int id=-1;
   size_t len;
   unsigned char *value_e;
   size_t value_e_len;
   unsigned char *language_e=NULL;
   size_t language_e_len;
   int datatype_id= -1;
-  raptor_stringbuffer *sb;
+  raptor_stringbuffer *sb=NULL;
   unsigned char *expression;
 
   value_e=sqlite_string_escape(value, value_len, &value_e_len);
   if(!value_e)
-    return -1;
+    goto tidy;
 
   sb=raptor_new_stringbuffer();
+  if(!sb)
+    goto tidy;
 
   raptor_stringbuffer_append_counted_string(sb,  
                                             (const unsigned char*)"text = ",
@@ -561,7 +603,7 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
     len=strlen(language);
     language_e=sqlite_string_escape((unsigned const char*)language, len, &language_e_len);
     if(!language_e)
-      return -1;
+      goto tidy;
     raptor_stringbuffer_append_string(sb, (const unsigned char*)"AND language = ", 1);
     raptor_stringbuffer_append_counted_string(sb, language_e, language_e_len, 1);
   } else
@@ -576,12 +618,16 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
   
   expression=raptor_stringbuffer_as_string(sb);
   id=librdf_storage_sqlite_get_helper(storage, TABLE_LITERALS, expression);
-  raptor_free_stringbuffer(sb);
   
   if(id >=0)
     goto tidy;
   
+  raptor_free_stringbuffer(sb);
   sb=raptor_new_stringbuffer();
+  if(!sb) {
+    id=-1;
+    goto tidy;
+  }
 
   raptor_stringbuffer_append_counted_string(sb, value_e, value_e_len, 1);
 
@@ -600,11 +646,12 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
   expression=raptor_stringbuffer_as_string(sb);
   id=librdf_storage_sqlite_set_helper(storage, TABLE_LITERALS, expression,
                                       raptor_stringbuffer_length(sb));
-  raptor_free_stringbuffer(sb);
 
   tidy:
-  LIBRDF_FREE(cstring, value_e);
-
+  if(sb)
+    raptor_free_stringbuffer(sb);
+  if(value_e)
+    LIBRDF_FREE(cstring, value_e);
   if(language_e)
     LIBRDF_FREE(cstring, language_e);
   
@@ -745,9 +792,7 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
 #if SQLITE_API == 2
     sqlite_FREE(errmsg);
 #endif
-#if SQLITE_API == 3
-    sqlite_CLOSE(context->db);
-#endif
+    librdf_storage_sqlite_close(storage);
     return 1;
   }
 
@@ -757,6 +802,11 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
     unsigned char *request;
 
     sb=raptor_new_stringbuffer();
+    if(!sb) {
+      librdf_storage_sqlite_close(storage);
+      return 1;
+    }
+
     raptor_stringbuffer_append_string(sb, 
                                       (const unsigned char*)"PRAGMA synchronous=", 1);
     raptor_stringbuffer_append_string(sb, 
@@ -765,10 +815,14 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
     
     request=raptor_stringbuffer_as_string(sb);
 
-    librdf_storage_sqlite_exec(storage, 
-                               request,
-                               NULL, NULL, 0);
+    rc=librdf_storage_sqlite_exec(storage, 
+                                  request,
+                                  NULL, NULL, 0);
     raptor_free_stringbuffer(sb);
+    if(rc) {
+      librdf_storage_sqlite_close(storage);
+      return 1;
+    }
   }
 
   
@@ -777,9 +831,7 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
     unsigned char request[200];
     int begin;
 
-    begin=librdf_storage_sqlite_exec(storage,
-                                     (unsigned char *)"BEGIN TRANSACTION;",
-                                     NULL, NULL, 0);
+    begin=librdf_storage_sqlite_transaction_start(storage);
 
     for(i=0; i < NTABLES; i++) {
 
@@ -801,9 +853,8 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
                                     NULL, /* arg */
                                     0)) {
         if(!begin)
-          librdf_storage_sqlite_exec(storage,
-                                     (unsigned char *) "END TRANSACTION;",
-                                     NULL, NULL, 0);
+          librdf_storage_sqlite_transaction_rollback(storage);
+        librdf_storage_sqlite_close(storage);
         return 1;
       }
 
@@ -817,9 +868,8 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
                                   NULL, /* arg */
                                   0)) {
       if(!begin)
-        librdf_storage_sqlite_exec(storage,
-                                   (unsigned char *) "END TRANSACTION;",
-                                   NULL, NULL, 0);
+        librdf_storage_sqlite_transaction_rollback(storage);
+      librdf_storage_sqlite_close(storage);
       return 1;
     }
     
@@ -831,17 +881,13 @@ librdf_storage_sqlite_open(librdf_storage* storage, librdf_model* model)
                                   NULL, /* arg */
                                   0)) {
       if(!begin)
-        librdf_storage_sqlite_exec(storage,
-                                   (unsigned char *) "END TRANSACTION;",
-                                   NULL, NULL, 0);
+        librdf_storage_sqlite_transaction_rollback(storage);
+      librdf_storage_sqlite_close(storage);
       return 1;
     }
     
     if(!begin)
-      librdf_storage_sqlite_exec(storage,
-                                 (unsigned char *) "END TRANSACTION;",
-                                 NULL, NULL, 0);
-    
+      librdf_storage_sqlite_transaction_commit(storage);    
   } /* end if is new */
 
   return 0;
@@ -905,6 +951,10 @@ librdf_storage_sqlite_add_statements(librdf_storage* storage,
 {
   /* librdf_storage_sqlite_context* context=(librdf_storage_sqlite_context*)storage->context; */
   int status=0;
+  int begin;
+
+  /* returns non-0 if a transaction is already active */
+  begin=librdf_storage_sqlite_transaction_start(storage);
 
   for(; !librdf_stream_end(statement_stream);
       librdf_stream_next(statement_stream)) {
@@ -931,14 +981,22 @@ librdf_storage_sqlite_add_statements(librdf_storage* storage,
     if(librdf_storage_sqlite_statement_helper(storage,
                                               statement,
                                               context_node,
-                                              node_types, node_ids, fields))
+                                              node_types, node_ids, fields)) {
+      if(!begin)
+        librdf_storage_sqlite_transaction_rollback(storage);
       return -1;
+    }
     
     if(context_node)
       max++;
     
     /* FIXME no context field used */
     sb=raptor_new_stringbuffer();
+    if(!sb) {
+      if(!begin)
+        librdf_storage_sqlite_transaction_rollback(storage);
+      return -1;
+    }
     raptor_stringbuffer_append_string(sb,
                                       (unsigned char*)"INSERT INTO ", 1);
     raptor_stringbuffer_append_string(sb, 
@@ -973,10 +1031,16 @@ librdf_storage_sqlite_add_statements(librdf_storage* storage,
 
     raptor_free_stringbuffer(sb);
 
-    if(rc)
+    if(rc) {
+      if(!begin)
+        librdf_storage_sqlite_transaction_rollback(storage);
       return 1;
+    }
 
   }
+
+  if(!begin)
+    librdf_storage_sqlite_transaction_commit(storage);
   
   return status;
 }
@@ -1046,20 +1110,29 @@ librdf_storage_sqlite_contains_statement(librdf_storage* storage,
   int count=0;
   int rc, begin;
 
-  begin=librdf_storage_sqlite_exec(storage,
-                                   (unsigned char *) "BEGIN TRANSACTION;",
-                                   NULL, NULL, 0);
-
   sb=raptor_new_stringbuffer();
+  if(!sb)
+    return -1;
+
+  /* returns non-0 if a transaction is already active */
+  begin=librdf_storage_sqlite_transaction_start(storage);
+
   raptor_stringbuffer_append_string(sb, 
-                                    (const unsigned char*)"SELECT subjectUri",
+                                    (const unsigned char*)"SELECT 1",
                                     1);
+  /* FIXME:
+   * librdf_storage_sqlite_statement_operator_helper()
+   * calls librdf_storage_sqlite_statement_helper()
+   * which calls librdf_storage_sqlite_node_helper()
+   * which calls librdf_storage_sqlite_{uri,literal,blank}_helper() 
+   * which call librdf_storage_sqlite_set_helper() 
+   * which creates new rows for unseen uris/literals/blanks
+   * - should not need to insert new data if merely testing for existence.
+   */
   if(librdf_storage_sqlite_statement_operator_helper(storage, statement, 
                                                      NULL, sb)) {
     if(!begin)
-      librdf_storage_sqlite_exec(storage,
-                                 (unsigned char *) "END TRANSACTION;",
-                                 NULL, NULL, 0);
+      librdf_storage_sqlite_transaction_rollback(storage);
     raptor_free_stringbuffer(sb);
     return -1;
   }
@@ -1076,9 +1149,7 @@ librdf_storage_sqlite_contains_statement(librdf_storage* storage,
   raptor_free_stringbuffer(sb);
 
   if(!begin)
-    librdf_storage_sqlite_exec(storage,
-                               (unsigned char *) "END TRANSACTION;",
-                               NULL, NULL, 0);
+    librdf_storage_transaction_commit(storage);
 
   if(rc)  
     return -1;
@@ -1159,15 +1230,27 @@ librdf_storage_sqlite_serialise(librdf_storage* storage)
   if(!scontext)
     return NULL;
 
+  scontext->storage=storage;
+  librdf_storage_add_reference(scontext->storage);
+
   scontext->sqlite_context=context;
   context->in_stream++;
 
   sb=raptor_new_stringbuffer();
+  if(!sb) {
+    librdf_storage_sqlite_serialise_finished((void*)scontext);
+    return NULL;
+  }
   sqlite_construct_select_helper(sb);
   raptor_stringbuffer_append_counted_string(sb, 
                                             (unsigned char*)";", 1, 1);
 
   request=raptor_stringbuffer_as_string(sb);
+  if(!request) {
+    raptor_free_stringbuffer(sb);
+    librdf_storage_sqlite_serialise_finished((void*)scontext);
+    return NULL;
+  }
 
   LIBRDF_DEBUG2("SQLite prepare '%s'\n", request);
 
@@ -1188,20 +1271,17 @@ librdf_storage_sqlite_serialise(librdf_storage* storage)
                         &errmsg);
 #endif
 
+  raptor_free_stringbuffer(sb);
+
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile failed - %s (%d)", 
                context->name, errmsg, status);
 
-    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_serialise_finished((void*)scontext);
     return NULL;
   }
-  raptor_free_stringbuffer(sb);
   
-  scontext->storage=storage;
-  librdf_storage_add_reference(scontext->storage);
-
   stream=librdf_new_stream(storage->world,
                            (void*)scontext,
                            &librdf_storage_sqlite_serialise_end_of_stream,
@@ -1306,12 +1386,18 @@ librdf_storage_sqlite_get_next_common(librdf_storage_sqlite_context* scontext,
       node=librdf_new_node_from_blank_identifier(scontext->storage->world,
                                                  blank);
     }
+    if(!node)
+      /* finished on error */
+      return 1;
     librdf_statement_set_subject(*statement, node);
 
 
     uri_string=GET_COLUMN_VALUE_TEXT(vm, 2);
     node=librdf_new_node_from_uri_string(scontext->storage->world,
                                          uri_string);
+    if(!node)
+      /* finished on error */
+      return 1;
     librdf_statement_set_predicate(*statement, node);
 
     uri_string=GET_COLUMN_VALUE_TEXT(vm, 3);
@@ -1329,8 +1415,12 @@ librdf_storage_sqlite_get_next_common(librdf_storage_sqlite_context* scontext,
       
       /* int datatype_id= GET_COLUMN_VALUE_INT(vm, 7); */
       uri_string=GET_COLUMN_VALUE_TEXT(vm, 8);
-      if(uri_string)
+      if(uri_string) {
         datatype=librdf_new_uri(scontext->storage->world, uri_string);
+        if(!datatype)
+          /* finished on error */
+          return 1;
+      }
       
       node=librdf_new_node_from_typed_literal(scontext->storage->world,
                                               literal, 
@@ -1340,12 +1430,20 @@ librdf_storage_sqlite_get_next_common(librdf_storage_sqlite_context* scontext,
         librdf_free_uri(datatype);
       
     }
+    if(!node)
+      /* finished on error */
+      return 1;
     librdf_statement_set_object(*statement, node);
 
     uri_string=GET_COLUMN_VALUE_TEXT(vm, 9);
-    if(uri_string)
-      *context_node=librdf_new_node_from_uri_string(scontext->storage->world,
-                                                    uri_string);
+    if(uri_string) {
+      node=librdf_new_node_from_uri_string(scontext->storage->world,
+                                           uri_string);
+      if(!node)
+        /* finished on error */
+        return 1;
+      *context_node=node;
+    }
   }
 
   if(status != SQLITE_ROW)
@@ -1547,10 +1645,12 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
   if(!scontext)
     return NULL;
 
+  scontext->storage=storage;
+  librdf_storage_add_reference(scontext->storage);
+
   scontext->sqlite_context=context;
   context->in_stream++;
 
-
   scontext->query_statement=librdf_new_statement_from_statement(statement);
   if(!scontext->query_statement) {
     librdf_storage_sqlite_find_statements_finished((void*)scontext);
@@ -1566,6 +1666,11 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
   }
 
   sb=raptor_new_stringbuffer();
+  if(!sb) {
+    librdf_storage_sqlite_find_statements_finished((void*)scontext);
+    return NULL;
+  }
+
   sqlite_construct_select_helper(sb);
 
   for(i=0; i < 3; i++) {
@@ -1593,7 +1698,12 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
                                             (unsigned char*)";", 1, 1);
   
   request=raptor_stringbuffer_as_string(sb);
-  
+  if(!request) {
+    raptor_free_stringbuffer(sb);
+    librdf_storage_sqlite_find_statements_finished((void*)scontext);
+    return NULL;
+  }
+
   LIBRDF_DEBUG2("SQLite prepare '%s'\n", request);
 
 #if SQLITE_API == 3
@@ -1613,20 +1723,17 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
                         &errmsg);
 #endif
 
+  raptor_free_stringbuffer(sb);
+
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile '%s' failed - %s (%d)", 
                context->name, request, errmsg, status);
 
-    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_find_statements_finished((void*)scontext);
     return NULL;
   }
-  raptor_free_stringbuffer(sb);
   
-  scontext->storage=storage;
-  librdf_storage_add_reference(scontext->storage);
-
   stream=librdf_new_stream(storage->world,
                            (void*)scontext,
                            &librdf_storage_sqlite_find_statements_end_of_stream,
@@ -1784,10 +1891,13 @@ librdf_storage_sqlite_context_add_statement(librdf_storage* storage,
   int i;
   int rc, begin;
   int max=3;
-  
-  begin=librdf_storage_sqlite_exec(storage,
-                                   (unsigned char *) "BEGIN TRANSACTION;",
-                                   NULL, NULL, 0);
+
+  sb=raptor_new_stringbuffer();
+  if(!sb)
+    return -1;
+
+  /* returns non-0 if transaction is already active */
+  begin=librdf_storage_sqlite_transaction_start(storage);
 
   if(librdf_storage_sqlite_statement_helper(storage,
                                             statement,
@@ -1795,16 +1905,14 @@ librdf_storage_sqlite_context_add_statement(librdf_storage* storage,
                                             node_types, node_ids, fields)) {
 
     if(!begin)
-      librdf_storage_sqlite_exec(storage,
-                                 (unsigned char *) "END TRANSACTION;",
-                                 NULL, NULL, 0);
+      librdf_storage_sqlite_transaction_rollback(storage);
+    raptor_free_stringbuffer(sb);
     return -1;
   }
   
   if(context_node)
     max++;
 
-  sb=raptor_new_stringbuffer();
   raptor_stringbuffer_append_string(sb, 
                                     (unsigned char*)"INSERT INTO ", 1);
   raptor_stringbuffer_append_string(sb, 
@@ -1839,14 +1947,14 @@ librdf_storage_sqlite_context_add_statement(librdf_storage* storage,
   
   raptor_free_stringbuffer(sb);
   
+  if(rc) {
+    if(!begin)
+      librdf_storage_transaction_rollback(storage);
+    return rc;
+  }
+
   if(!begin)
-    librdf_storage_sqlite_exec(storage,
-                               (unsigned char *) "END TRANSACTION;",
-                               NULL, NULL, 0);
-
-  if(rc)
-    return 1;
-
+    librdf_storage_transaction_commit(storage);
   return 0;
 }
 
@@ -1872,6 +1980,8 @@ librdf_storage_sqlite_context_remove_statement(librdf_storage* storage,
   unsigned char *request;
 
   sb=raptor_new_stringbuffer();
+  if(!sb)
+    return -1;
   raptor_stringbuffer_append_string(sb, (const unsigned char*)"DELETE", 1);
   if(librdf_storage_sqlite_statement_operator_helper(storage, statement,
                                                      context_node, sb)) {
@@ -1915,6 +2025,8 @@ librdf_storage_sqlite_context_remove_statements(librdf_storage* storage,
     return -1;
     
   sb=raptor_new_stringbuffer();
+  if(!sb)
+    return -1;
   raptor_stringbuffer_append_counted_string(sb,
                                     (unsigned char*)"DELETE FROM ", 12, 1);
   raptor_stringbuffer_append_string(sb, 
@@ -1994,6 +2106,9 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
   if(!scontext)
     return NULL;
 
+  scontext->storage=storage;
+  librdf_storage_add_reference(scontext->storage);
+
   scontext->sqlite_context=context;
   context->in_stream++;
 
@@ -2008,6 +2123,10 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
   }
 
   sb=raptor_new_stringbuffer();
+  if(!sb) {
+    librdf_storage_sqlite_context_serialise_finished((void*)scontext);
+    return NULL;
+  }
   sqlite_construct_select_helper(sb);
   raptor_stringbuffer_append_counted_string(sb, 
                                             (unsigned char*)" WHERE ", 7, 1);
@@ -2023,6 +2142,11 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
                                             (unsigned char*)";", 1, 1);
   
   request=raptor_stringbuffer_as_string(sb);
+  if(!request) {
+    raptor_free_stringbuffer(sb);
+    librdf_storage_sqlite_context_serialise_finished((void*)scontext);
+    return NULL;
+  }
 
   LIBRDF_DEBUG2("SQLite prepare '%s'\n", request);
 
@@ -2043,19 +2167,16 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
                         &errmsg);
 #endif
 
+  raptor_free_stringbuffer(sb);
+
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile failed - %s (%d)", 
                context->name, errmsg, status);
 
-    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_context_serialise_finished((void*)scontext);
     return NULL;
   }
-  raptor_free_stringbuffer(sb);
-
-  scontext->storage=storage;
-  librdf_storage_add_reference(scontext->storage);
 
   stream=librdf_new_stream(storage->world,
                            (void*)scontext,
@@ -2260,10 +2381,15 @@ librdf_storage_sqlite_get_next_context_common(librdf_storage_sqlite_context* sco
 
     uri_string=GET_COLUMN_VALUE_TEXT(vm, 0);
     if(uri_string) {
+      librdf_node *node;
+      node=librdf_new_node_from_uri_string(scontext->storage->world,
+                                           uri_string);
+      if(!node)
+        /* finished on error */
+        return 1;
       if(*context_node)
-        librdf_free_node(*context_node);      
-      *context_node=librdf_new_node_from_uri_string(scontext->storage->world,
-                                                    uri_string);
+        librdf_free_node(*context_node);
+      *context_node=node;
     }
   }
 
@@ -2427,11 +2553,11 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
 {
   librdf_storage_sqlite_context* context=(librdf_storage_sqlite_context*)storage->context;
   librdf_storage_sqlite_get_contexts_iterator_context* icontext;
-  librdf_iterator* iterator;
   int status;
   char *errmsg=NULL;
   raptor_stringbuffer *sb;
   unsigned char *request;
+  librdf_iterator* iterator;
 
   icontext=(librdf_storage_sqlite_get_contexts_iterator_context*)LIBRDF_CALLOC(librdf_storage_sqlite_get_contexts_iterator_context, 1, sizeof(librdf_storage_sqlite_get_contexts_iterator_context));
   if(!icontext)
@@ -2440,6 +2566,10 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
   icontext->sqlite_context=context;
 
   sb=raptor_new_stringbuffer();
+  if(!sb) {
+    LIBRDF_FREE(librdf_storage_sqlite_get_contexts_iterator_context, icontext);
+    return NULL;
+  }
 
   raptor_stringbuffer_append_string(sb, (unsigned char*)
                                     "SELECT DISTINCT uris.uri", 1);
@@ -2451,7 +2581,12 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
                                             (const unsigned char*)" LEFT JOIN uris ON uris.id = contextUri WHERE contextUri NOT NULL;", 1);
 
   request=raptor_stringbuffer_as_string(sb);
-  
+  if(!request) {
+    raptor_free_stringbuffer(sb);
+    LIBRDF_FREE(librdf_storage_sqlite_get_contexts_iterator_context, icontext);
+    return NULL;
+  }
+
   LIBRDF_DEBUG2("SQLite prepare '%s'\n", request);
 
 #if SQLITE_API == 3
@@ -2471,16 +2606,16 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
                         &errmsg);
 #endif
 
+  raptor_free_stringbuffer(sb);
+
   if(status != SQLITE_OK) {
     librdf_log(storage->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "SQLite database %s SQL compile failed - %s (%d)", 
                context->name, errmsg, status);
 
-    raptor_free_stringbuffer(sb);
     librdf_storage_sqlite_get_contexts_finished((void*)icontext);
     return NULL;
   }
-  raptor_free_stringbuffer(sb);
     
   icontext->storage=storage;
   librdf_storage_add_reference(icontext->storage);
@@ -2492,9 +2627,8 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
                                &librdf_storage_sqlite_get_contexts_get_method,
                                &librdf_storage_sqlite_get_contexts_finished);
   if(!iterator)
-    librdf_storage_sqlite_get_contexts_finished((void*)icontext);
-  
-  return iterator;  
+    librdf_storage_sqlite_get_contexts_finished(icontext);
+  return iterator;
 }
 
 
@@ -2531,6 +2665,79 @@ librdf_storage_sqlite_get_feature(librdf_storage* storage, librdf_uri* feature)
   return NULL;
 }
 
+
+/**
+ * librdf_storage_sqlite_transaction_start:
+ * @storage: #librdf_storage object
+ *
+ * Start a new transaction unless one is already active.
+ * 
+ * Return value: 0 if transaction successfully started, non-0 on error
+ * (including a transaction already active)
+ **/
+static int
+librdf_storage_sqlite_transaction_start(librdf_storage *storage)
+{
+  librdf_storage_sqlite_context *context=(librdf_storage_sqlite_context *)storage->context;
+
+  if(context->in_transaction)
+    return 1;
+  context->in_transaction++;
+
+  return librdf_storage_sqlite_exec(storage,
+                                    (unsigned char *)"BEGIN;",
+                                    NULL, NULL, 0);
+}
+
+
+/**
+ * librdf_storage_sqlite_transaction_commit:
+ * @storage: #librdf_storage object
+ *
+ * Commit an active transaction.
+ * 
+ * Return value: 0 if transaction successfully committed, non-0 on error
+ * (including no transaction active)
+ **/
+static int
+librdf_storage_sqlite_transaction_commit(librdf_storage *storage)
+{
+  librdf_storage_sqlite_context *context=(librdf_storage_sqlite_context *)storage->context;
+
+  if(!context->in_transaction)
+    return 1;
+  context->in_transaction--;
+
+  return librdf_storage_sqlite_exec(storage,
+                                    (unsigned char *)"END",
+                                    NULL, NULL, 0);
+}
+
+
+/**
+ * librdf_storage_sqlite_transaction_rollback:
+ * @storage: #librdf_storage object
+ *
+ * Roll back an active transaction.
+ * 
+ * Return value: 0 if transaction successfully committed, non-0 on error
+ * (including no transaction active)
+ **/
+static int
+librdf_storage_sqlite_transaction_rollback(librdf_storage *storage)
+{
+  librdf_storage_sqlite_context *context=(librdf_storage_sqlite_context *)storage->context;
+
+  if(!context->in_transaction)
+    return 1;
+  context->in_transaction--;
+
+  return librdf_storage_sqlite_exec(storage,
+                                    (unsigned char *)"ROLLBACK;",
+                                    NULL, NULL, 0);
+}
+
+
 static void
 librdf_storage_sqlite_query_flush(librdf_storage *storage)
 {
@@ -2541,9 +2748,8 @@ librdf_storage_sqlite_query_flush(librdf_storage *storage)
   if(!context->in_stream_queries)
     return;
 
-  begin=librdf_storage_sqlite_exec(storage,
-                                   (unsigned char *) "BEGIN TRANSACTION;",
-                                   NULL, NULL, 0);
+  /* returns non-0 if a transaction is already active */  
+  begin=librdf_storage_sqlite_transaction_start(storage);
 
   while(context->in_stream_queries) {
     query=context->in_stream_queries;
@@ -2556,9 +2762,7 @@ librdf_storage_sqlite_query_flush(librdf_storage *storage)
   }
 
   if(!begin)
-    librdf_storage_sqlite_exec(storage,
-                               (unsigned char *) "END TRANSACTION;",
-                               NULL, NULL, 0);
+    librdf_storage_sqlite_transaction_commit(storage);
 }
 
 /* local function to register sqlite storage functions */
@@ -2585,6 +2789,9 @@ librdf_storage_sqlite_register_factory(librdf_storage_factory *factory)
   factory->context_serialise        = librdf_storage_sqlite_context_serialise;
   factory->get_contexts             = librdf_storage_sqlite_get_contexts;
   factory->get_feature              = librdf_storage_sqlite_get_feature;
+  factory->transaction_start        = librdf_storage_sqlite_transaction_start;
+  factory->transaction_commit       = librdf_storage_sqlite_transaction_commit;
+  factory->transaction_rollback     = librdf_storage_sqlite_transaction_rollback;
 }
 
 
