@@ -2,7 +2,7 @@
  *
  * rdf_storage_tstore.c - RDF Storage using 3store
  *
- * $Id: rdf_storage_tstore.c 8534 2006-01-26 04:57:33Z cmdjb $
+ * $Id: rdf_storage_tstore.c 11930 2007-02-25 19:46:53Z dajobe $
  *
  * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2004, University of Bristol, UK http://www.bristol.ac.uk/
@@ -59,7 +59,7 @@ typedef struct
 
 
 /* prototypes for local functions */
-static int librdf_storage_tstore_init(librdf_storage* storage, char *name, librdf_hash* options);
+static int librdf_storage_tstore_init(librdf_storage* storage, const char *name, librdf_hash* options);
 static int librdf_storage_tstore_open(librdf_storage* storage, librdf_model* model);
 static int librdf_storage_tstore_close(librdf_storage* storage);
 static int librdf_storage_tstore_size(librdf_storage* storage);
@@ -104,8 +104,8 @@ static void librdf_storage_tstore_register_factory(librdf_storage_factory *facto
 
 /* functions implementing storage api */
 static int
-librdf_storage_tstore_init(librdf_storage* storage, char *name,
-                         librdf_hash* options)
+librdf_storage_tstore_init(librdf_storage* storage, const char *name,
+                           librdf_hash* options)
 {
   librdf_storage_tstore_context *context=(librdf_storage_tstore_context*)storage->context;
   
@@ -731,6 +731,12 @@ librdf_storage_tstore_register_factory(librdf_storage_factory *factory)
 }
 
 
+/**
+ * librdf_init_storage_tstore:
+ * @world: world object
+ * 
+ * INTERNAL - initialise the storage_tstore module.
+ **/
 void
 librdf_init_storage_tstore(librdf_world *world)
 {
