@@ -34,11 +34,12 @@ extern "C" {
 #ifdef RAPTOR_INTERNAL
 
 /* for the memory allocation functions */
-#if defined(HAVE_DMALLOC_H) && defined(RAPTOR_MEMORY_DEBUG_DMALLOC)
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #undef HAVE_STDLIB_H
 #endif
+
+#if defined(HAVE_DMALLOC_H) && defined(RAPTOR_MEMORY_DEBUG_DMALLOC)
 #include <dmalloc.h>
 #endif
 
@@ -91,7 +92,9 @@ void raptor_system_free(void *ptr);
 #define SYSTEM_FREE(ptr)   free(ptr)
 #endif
 
+#ifndef RAPTOR_ASSERT_DIE
 #define RAPTOR_ASSERT_DIE abort();
+#endif
 
 #else
 /* DEBUGGING TURNED OFF */
@@ -107,7 +110,9 @@ void raptor_system_free(void *ptr);
 #define SYSTEM_MALLOC(size)   malloc(size)
 #define SYSTEM_FREE(ptr)   free(ptr)
 
+#ifndef RAPTOR_ASSERT_DIE
 #define RAPTOR_ASSERT_DIE
+#endif
 
 #endif
 
@@ -193,7 +198,7 @@ typedef struct raptor_sax2_s raptor_sax2;
 /* raptor_libxml.c exports */
 extern void raptor_libxml_init(raptor_sax2* sax2, raptor_uri *base_uri);
 extern void raptor_libxml_init_sax_error_handlers(xmlSAXHandler *sax);
-extern void raptor_libxml_init_generic_error_handlers(raptor_sax2 *sax2);
+extern void raptor_libxml_generic_error(void* user_data, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 
 extern void raptor_libxml_validation_error(void *context, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 extern void raptor_libxml_validation_warning(void *context, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
@@ -227,6 +232,12 @@ typedef struct raptor_serializer_factory_s raptor_serializer_factory;
 typedef struct raptor_id_set_s raptor_id_set;
 typedef struct raptor_uri_detail_s raptor_uri_detail;
 
+/* raptor_avltree.c */
+/* AVL data */
+typedef void* raptor_avltree_t;
+/* AVL tree */
+typedef struct raptor_avltree_s raptor_avltree;
+
 
 /* Raptor Namespace Stack node */
 struct raptor_namespace_stack_s {
@@ -271,19 +282,6 @@ struct raptor_namespace_s {
 };
 
 
-typedef struct {
-  void *fatal_error_user_data;
-  void *error_user_data;
-  void *warning_user_data;
-
-  raptor_message_handler fatal_error_handler;
-  raptor_message_handler error_handler;
-  raptor_message_handler warning_handler;
-
-  raptor_locator* locator;
-} raptor_error_handlers;
-
-
 typedef enum {
   RAPTOR_LOG_LEVEL_NONE,
   RAPTOR_LOG_LEVEL_FATAL,
@@ -293,6 +291,17 @@ typedef enum {
 } raptor_log_level;
 
 
+#define RAPTOR_ERROR_HANDLER_MAGIC 0xD00DB1FF
+typedef struct {
+  unsigned int magic;
+
+  raptor_locator* locator;
+
+  void* user_data[RAPTOR_LOG_LEVEL_LAST+1];
+  raptor_message_handler handlers[RAPTOR_LOG_LEVEL_LAST+1];
+} raptor_error_handlers;
+
+
 #ifdef RAPTOR_XML_LIBXML
 #define RAPTOR_LIBXML_MAGIC 0x8AF108
 #endif
@@ -334,6 +343,8 @@ struct raptor_parser_s {
   /* parser callbacks */
   raptor_statement_handler statement_handler;
 
+  raptor_graph_handler graph_handler;
+
   void *generate_id_handler_user_data;
   raptor_generate_id_handler generate_id_handler;
 
@@ -418,6 +429,9 @@ struct raptor_parser_factory_s {
 
   /* non-0 if this parser needs a base URI */
   int need_base_uri;
+
+  /* get the current generated ID base (OPTIONAL) */
+  int (*get_current_base_id)(raptor_parser* rdf_parser);
 };
 
 
@@ -528,6 +542,18 @@ struct raptor_serializer_factory_s {
 };
 
 
+/* for raptor_parse_uri_write_bytes() when used as a handler for
+ * raptor_www_set_write_bytes_handler()
+ */
+typedef struct 
+{
+  raptor_parser* rdf_parser;
+  raptor_uri* base_uri;
+  raptor_uri* final_uri;
+  int started;
+} raptor_parse_bytes_context;
+
+
 /* raptor_serialize.c */
 void raptor_serializer_register_factory(const char *name, const char *label, const char *mime_type, const char *alias, const unsigned char *uri_string, void (*factory) (raptor_serializer_factory*));
 
@@ -554,6 +580,7 @@ void raptor_delete_parser_factories(void);
 void raptor_free_type_q(raptor_type_q* type_q);
 const char* raptor_parser_get_accept_header_all(void);
 int raptor_parse_uri_no_net_filter(void *user_data, raptor_uri* uri);
+void raptor_parse_uri_write_bytes(raptor_www* www, void *userdata, const void *ptr, size_t size, size_t nmemb);
 
 /* raptor_general.c */
 extern void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
@@ -586,7 +613,7 @@ extern void raptor_xml_characters_handler(void *user_data, const unsigned char *
 extern void raptor_xml_cdata_handler(void *user_data, const unsigned char *s, int len);
 void raptor_xml_comment_handler(void *user_data, const unsigned char *s);
 
-#ifdef RAPTOR_DEBUG
+#if RAPTOR_DEBUG > 1
 void raptor_rdfxml_parser_stats_print(raptor_rdfxml_parser* rdf_xml_parser, FILE *stream);
 #endif
 
@@ -666,12 +693,14 @@ raptor_uri* raptor_new_uri_from_rdf_ordinal(int ordinal);
 void raptor_init_parser_rdfxml(void);
 void raptor_init_parser_ntriples(void);
 void raptor_init_parser_turtle(void);
+void raptor_init_parser_trig(void);
 void raptor_init_parser_n3(void);
+void raptor_init_parser_grddl_common(void);
 void raptor_init_parser_grddl(void);
 void raptor_init_parser_guess(void);
 void raptor_init_parser_rss(void);
 
-void raptor_terminate_parser_grddl(void);
+void raptor_terminate_parser_grddl_common(void);
 
 /* raptor_parse.c */
 void raptor_parsers_init(void);
@@ -679,7 +708,8 @@ void raptor_parsers_finish(void);
 
 void raptor_parser_save_content(raptor_parser* rdf_parser, int save);
 const unsigned char* raptor_parser_get_content(raptor_parser* rdf_parser, size_t* length_p);
-
+void raptor_parser_set_graph_name(raptor_parser* parser, raptor_uri* uri);
+int raptor_parser_get_current_base_id(raptor_parser* parser);
 
 /* raptor_rss.c */
 void raptor_init_serializer_rss10(void);
@@ -714,10 +744,10 @@ void raptor_init_serializer_rdfxmla(void);
 
 /* raptor_serialize_turtle.c */  
 void raptor_init_serializer_turtle(void);
-  
+
 /* raptor_utf8.c */
-int raptor_unicode_is_namestartchar(long c);
-int raptor_unicode_is_namechar(long c);
+int raptor_unicode_is_namestartchar(raptor_unichar c);
+int raptor_unicode_is_namechar(raptor_unichar c);
 int raptor_utf8_is_nfc(const unsigned char *input, size_t length);
 
 /* raptor_www*.c */
@@ -734,7 +764,9 @@ int raptor_utf8_is_nfc(const unsigned char *input, size_t length);
 #endif
 
 /* Size of buffer used in various raptor_www places for I/O  */
+#ifndef RAPTOR_WWW_BUFFER_SIZE
 #define RAPTOR_WWW_BUFFER_SIZE 4096
+#endif
 
 /* WWW library state */
 struct  raptor_www_s {
@@ -750,13 +782,14 @@ struct  raptor_www_s {
   CURL* curl_handle;
   char error_buffer[CURL_ERROR_SIZE];
   int curl_init_here;
+  int checked_status;
 #endif
 
 #ifdef RAPTOR_WWW_LIBXML
   void *ctxt;
-  xmlGenericErrorFunc old_handler;
   char buffer[RAPTOR_WWW_BUFFER_SIZE];
   int is_end;
+  void *old_xmlGenericErrorContext;
 #endif
 
 #ifdef RAPTOR_WWW_LIBFETCH
@@ -773,9 +806,6 @@ struct  raptor_www_s {
   void *content_type_userdata;
   raptor_www_content_type_handler content_type;
 
-  void *error_data;
-  raptor_message_handler error_handler;
-
   void* uri_filter_user_data;
   raptor_uri_filter_func uri_filter;
 
@@ -785,6 +815,16 @@ struct  raptor_www_s {
   char *http_accept;
 
   FILE* handle;
+
+  raptor_error_handlers error_handlers;
+
+  int connection_timeout;
+
+  /* The URI returned after any redirections */
+  raptor_uri* final_uri;
+
+  void *final_uri_userdata;
+  raptor_www_final_uri_handler final_uri_handler;
 };
 
 
@@ -794,7 +834,6 @@ void raptor_www_libxml_init(raptor_www *www);
 void raptor_www_libxml_free(raptor_www *www);
 int raptor_www_libxml_fetch(raptor_www *www);
 
-void raptor_www_error_varargs(raptor_www *www, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
 void raptor_www_error(raptor_www *www, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 
 void raptor_www_curl_init(raptor_www *www);
@@ -805,11 +844,11 @@ void raptor_www_libfetch_init(raptor_www *www);
 void raptor_www_libfetch_free(raptor_www *www);
 int raptor_www_libfetch_fetch(raptor_www *www);
 
-  /* raptor_set.c */
+/* raptor_set.c */
 raptor_id_set* raptor_new_id_set(void);
 void raptor_free_id_set(raptor_id_set* set);
 int raptor_id_set_add(raptor_id_set* set, raptor_uri* base_uri, const unsigned char *item, size_t item_len);
-#ifdef RAPTOR_DEBUG
+#if RAPTOR_DEBUG > 1
 void raptor_id_set_stats_print(raptor_id_set* set, FILE *stream);
 #endif
 
@@ -1030,7 +1069,7 @@ struct raptor_uri_detail_s
 #define RAPTOR_PARSEDATE_FUNCTION parsedate
 #else
 #ifdef HAVE_RAPTOR_PARSE_DATE
-time_t raptor_parse_date(char *p, time_t *now);
+time_t raptor_parse_date(const char *p, time_t *now);
 #define RAPTOR_PARSEDATE_FUNCTION raptor_parse_date
 #else
 #ifdef HAVE_CURL_CURL_H
@@ -1041,7 +1080,7 @@ time_t raptor_parse_date(char *p, time_t *now);
 #endif
 
 /* turtle_common.c */
-int raptor_stringbuffer_append_turtle_string(raptor_stringbuffer* stringbuffer, unsigned char *text, size_t len, int delim, raptor_simple_message_handler error_handler, void *error_data);
+int raptor_stringbuffer_append_turtle_string(raptor_stringbuffer* stringbuffer, const unsigned char *text, size_t len, int delim, raptor_simple_message_handler error_handler, void *error_data);
 
 
 /* raptor_xsd.c */
@@ -1094,16 +1133,17 @@ typedef struct {
 
 raptor_abbrev_node* raptor_new_abbrev_node(raptor_identifier_type node_type, const void *node_data, raptor_uri *datatype, const unsigned char *language);
 void raptor_free_abbrev_node(raptor_abbrev_node* node);
+int raptor_abbrev_node_cmp(raptor_abbrev_node* node1, raptor_abbrev_node* node2);
 int raptor_abbrev_node_equals(raptor_abbrev_node* node1, raptor_abbrev_node* node2);
 int raptor_abbrev_node_matches(raptor_abbrev_node* node, raptor_identifier_type node_type, const void *node_data, raptor_uri *datatype, const unsigned char *language);
-raptor_abbrev_node* raptor_abbrev_node_lookup(raptor_sequence* nodes, raptor_identifier_type node_type, const void *node_value, raptor_uri *datatype, const unsigned char *language);
+raptor_abbrev_node* raptor_abbrev_node_lookup(raptor_avltree* nodes, raptor_identifier_type node_type, const void *node_value, raptor_uri *datatype, const unsigned char *language);
 
 raptor_abbrev_subject* raptor_new_abbrev_subject(raptor_abbrev_node* node);
 void raptor_free_abbrev_subject(raptor_abbrev_subject* subject);
 int raptor_abbrev_subject_add_property(raptor_abbrev_subject* subject, raptor_abbrev_node* predicate, raptor_abbrev_node* object);
 int raptor_abbrev_subject_add_list_element(raptor_abbrev_subject* subject, int ordinal, raptor_abbrev_node* object);
 raptor_abbrev_subject* raptor_abbrev_subject_find(raptor_sequence *sequence, raptor_identifier_type node_type, const void *node_data, int *idx);
-raptor_abbrev_subject* raptor_abbrev_subject_lookup(raptor_sequence* nodes, raptor_sequence* subjects, raptor_sequence* blanks, raptor_identifier_type node_type, const void *node_data);
+raptor_abbrev_subject* raptor_abbrev_subject_lookup(raptor_avltree* nodes, raptor_sequence* subjects, raptor_sequence* blanks, raptor_identifier_type node_type, const void *node_data);
 
 unsigned char *raptor_unique_id(unsigned char *base);
 
@@ -1123,13 +1163,14 @@ void raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer);
 void raptor_turtle_writer_raw(raptor_turtle_writer* turtle_writer, const unsigned char *s);
 void raptor_turtle_writer_raw_counted(raptor_turtle_writer* turtle_writer, const unsigned char *s, unsigned int len);
 void raptor_turtle_writer_namespace_prefix(raptor_turtle_writer* turtle_writer, raptor_namespace* ns);
+void raptor_turtle_writer_base(raptor_turtle_writer* turtle_writer, raptor_uri* base_uri);
 void raptor_turtle_writer_increase_indent(raptor_turtle_writer *turtle_writer);
 void raptor_turtle_writer_decrease_indent(raptor_turtle_writer *turtle_writer);
 void raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer);
 void raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer, raptor_uri* uri);
-void raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer, unsigned char *s, unsigned char* lang, raptor_uri* datatype);
+void raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer, raptor_namespace_stack *nstack, const unsigned char *s, const unsigned char* lang, raptor_uri* datatype);
 void raptor_turtle_writer_qname(raptor_turtle_writer* turtle_writer, raptor_qname* qname);
-void raptor_turtle_writer_quoted(raptor_turtle_writer* turtle_writer, unsigned char *s);
+void raptor_turtle_writer_quoted(raptor_turtle_writer* turtle_writer, const unsigned char *s);
 void raptor_turtle_writer_comment(raptor_turtle_writer* turtle_writer, const unsigned char *s);
 int raptor_turtle_writer_features_enumerate(const raptor_feature feature, const char **name,  raptor_uri **uri, const char **label);
 int raptor_turtle_writer_set_feature(raptor_turtle_writer *turtle_writer, raptor_feature feature, int value);
@@ -1137,6 +1178,25 @@ int raptor_turtle_writer_set_feature_string(raptor_turtle_writer *turtle_writer,
 int raptor_turtle_writer_get_feature(raptor_turtle_writer *turtle_writer, raptor_feature feature);
 const unsigned char *raptor_turtle_writer_get_feature_string(raptor_turtle_writer *turtle_writer, raptor_feature feature);
 
+/* snprintf.c */
+char* raptor_format_float(char *buffer, size_t *currlen, size_t maxlen, double fvalue, unsigned int min, unsigned int max, int flags);
+
+/* user functions */
+typedef int (*raptor_avltree_compare_function)(raptor_avltree_t data1, raptor_avltree_t data2);
+typedef void (*raptor_avltree_delete_function)(raptor_avltree_t data);
+typedef int (*raptor_avltree_visit_function)(int depth, raptor_avltree_t data, void *user_data);
+
+/* constructor / destructor */
+raptor_avltree* raptor_new_avltree(raptor_avltree_compare_function compare_fn, raptor_avltree_delete_function delete_fn, unsigned int flags);
+void raptor_free_avltree(raptor_avltree* tree);
+
+/* methods */
+int raptor_avltree_add(raptor_avltree* tree, raptor_avltree_t p_user);
+int raptor_avltree_delete(raptor_avltree* tree, raptor_avltree_t);
+raptor_avltree_t raptor_avltree_search(raptor_avltree* tree, raptor_avltree_t p_user);
+int raptor_avltree_visit(raptor_avltree* tree, raptor_avltree_visit_function visit_fn, void* user_data);
+
+
 /* end of RAPTOR_INTERNAL */
 #endif
 
