@@ -66,6 +66,8 @@ raptor_sax2_finish(void)
 {
 #ifdef RAPTOR_XML_LIBXML
   xmlCleanupParser();
+  xmlSetStructuredErrorFunc(NULL, NULL);
+  xmlSetGenericErrorFunc(NULL, NULL);
 #endif
 }
 
@@ -89,8 +91,12 @@ raptor_new_sax2(void* user_data, raptor_error_handlers* error_handlers)
   sax2->error_handlers=error_handlers;
 
 #ifdef RAPTOR_XML_LIBXML
+  /* sets xmlGenericErrorContext and xmlStructuredError */
   xmlSetStructuredErrorFunc(&sax2->error_handlers, 
                             raptor_libxml_xmlStructuredErrorFunc);
+  /* sets xmlGenericErrorContext and xmlGenericError */
+  xmlSetGenericErrorFunc(&sax2->error_handlers, 
+                         (xmlGenericErrorFunc)raptor_libxml_generic_error);
 #endif
   
   return sax2;
@@ -251,6 +257,14 @@ raptor_xml_element_is_empty(raptor_xml_element* xml_element)
 }
 
 
+/**
+ * raptor_sax2_inscope_xml_language:
+ * @sax2: SAX2 object
+ * 
+ * Get the in-scope XML language
+ * 
+ * Return value: the XML language or NULL if none is in scope.
+ **/
 const unsigned char*
 raptor_sax2_inscope_xml_language(raptor_sax2 *sax2)
 {
@@ -259,8 +273,12 @@ raptor_sax2_inscope_xml_language(raptor_sax2 *sax2)
   for(xml_element=sax2->current_element;
       xml_element; 
       xml_element=xml_element->parent)
-    if(xml_element->xml_language)
+    if(xml_element->xml_language) {
+      if(!*xml_element->xml_language)
+        return NULL;
       return xml_element->xml_language;
+    }
+  
     
   return NULL;
 }
@@ -318,8 +336,8 @@ raptor_sax2_simple_error(void* user_data, const char *message, ...)
 
   if(sax2)
     raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
-                             sax2->error_handlers->error_handler,
-                             sax2->error_handlers->error_user_data,
+                             sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR],
+                             sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_ERROR],
                              sax2->locator,
                              message, arguments);
   
@@ -401,8 +419,8 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
       /* no data given at all - emit a similar message to expat */
       raptor_sax2_update_document_locator(sax2, sax2->locator);
       raptor_log_error_simple(RAPTOR_LOG_LEVEL_ERROR,
-                              sax2->error_handlers->error_handler,
-                              sax2->error_handlers->error_user_data,
+                              sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR],
+                              sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_ERROR],
                               sax2->locator,
                               "XML Parsing failed - no element found");
       return 1;
@@ -414,7 +432,7 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
     if(!xc)
       goto handle_error;
 
-#ifdef XML_PARSE_NONET
+#ifdef RAPTOR_LIBXML_XML_PARSE_NONET
     if(sax2->feature_no_net)
       libxml_options |= XML_PARSE_NONET;
 #endif
@@ -532,14 +550,14 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
       strncpy(error_buffer+ERROR_PREFIX_LEN, error_message, error_length+1);
 
       raptor_log_error_simple(RAPTOR_LOG_LEVEL_ERROR,
-                              sax2->error_handlers->error_handler,
-                              sax2->error_handlers->error_user_data,
+                              sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR],
+                              sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_ERROR],
                               sax2->locator, error_buffer);
       RAPTOR_FREE(cstring, error_buffer);
     } else
       raptor_log_error_simple(RAPTOR_LOG_LEVEL_ERROR,
-                              sax2->error_handlers->error_handler,
-                              sax2->error_handlers->error_user_data,
+                              sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR],
+                              sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_ERROR],
                               sax2->locator, "XML Parsing failed");
   }
 #endif
@@ -584,6 +602,10 @@ raptor_sax2_set_feature(raptor_sax2 *sax2, raptor_feature feature, int value)
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
     case RAPTOR_FEATURE_RELATIVE_URIS:
     case RAPTOR_FEATURE_START_URI:
     case RAPTOR_FEATURE_WRITER_AUTO_INDENT:
@@ -656,10 +678,10 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
 
       if (!dst) {
         raptor_log_error_simple(RAPTOR_LOG_LEVEL_FATAL,
-                                sax2->error_handlers->fatal_error_handler,
-                                sax2->error_handlers->fatal_error_user_data,
+                                sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL],
+                                sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_FATAL],
                                 sax2->locator, "Out of memory");
-	return;
+        return;
       }
 
       atts[i+1]=dst;
@@ -690,7 +712,7 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
     /* Save passed in XML attributes pointers so we can 
      * NULL the pointers when they get handled below (various atts[i]=NULL)
      */
-    for (i = 0; atts[i]; i++);
+    for (i = 0; atts[i]; i++) ;
     xml_atts_size=sizeof(unsigned char*) * i;
     if(xml_atts_size) {
       xml_atts_copy=(unsigned char**)RAPTOR_MALLOC(cstringpointer,xml_atts_size);
@@ -734,8 +756,8 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
         xml_language=(unsigned char*)RAPTOR_MALLOC(cstring, strlen((char*)atts[i+1])+1);
         if(!xml_language) {
           raptor_log_error_simple(RAPTOR_LOG_LEVEL_FATAL,
-                                  sax2->error_handlers->fatal_error_handler,
-                                  sax2->error_handlers->fatal_error_user_data,
+                                  sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL],
+                                  sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_FATAL],
                                   sax2->locator, "Out of memory");
           return;
         }
@@ -792,8 +814,8 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
                                               sizeof(raptor_qname*));
     if(!named_attrs) {
       raptor_log_error_simple(RAPTOR_LOG_LEVEL_FATAL,
-                              sax2->error_handlers->fatal_error_handler,
-                              sax2->error_handlers->fatal_error_user_data,
+                              sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL],
+                              sax2->error_handlers->user_data[RAPTOR_LOG_LEVEL_FATAL],
                               sax2->locator, "Out of memory");
       RAPTOR_FREE(raptor_xml_element, xml_element);
       raptor_free_qname(raptor_xml_element_get_name(xml_element));
