@@ -56,7 +56,7 @@
  * @xml_language: the in-scope XML language (or NULL)
  * @xml_base: the in-scope XML base URI (or NULL)
  * 
- * Constructor - create a new XML element
+ * Constructor - create a new XML element from a QName
  * 
  * Return value: a new #raptor_xml_element or NULL on failure
  **/
@@ -79,10 +79,49 @@ raptor_new_xml_element(raptor_qname *name,
 
   xml_element->declared_nspaces=NULL;
 
+  xml_element->content_cdata_sb=raptor_new_stringbuffer();
+
   return xml_element;
 }
 
 
+/**
+ * raptor_new_xml_element_from_namespace_local_name:
+ * @ns: namespace
+ * @name: the XML element local name
+ * @xml_language: the in-scope XML language (or NULL)
+ * @xml_base: base uri (or NULL)
+ *
+ * Constructor - create a new XML element from an XML namespace and a local name
+ *
+ * Added in 1.4.16.
+ *
+ * Return value: a new #raptor_xml_element or NULL on failure
+ */
+raptor_xml_element*
+raptor_new_xml_element_from_namespace_local_name(raptor_namespace *ns,
+                                                 const unsigned char *name,
+                                                 const unsigned char *xml_language, 
+                                                 raptor_uri *xml_base)
+{
+  raptor_uri *base_uri_copy;
+  raptor_qname *qname;
+  raptor_xml_element *element=NULL;
+
+  qname=raptor_new_qname_from_namespace_local_name(ns, name, NULL);
+  if(qname) {
+    base_uri_copy=xml_base ? raptor_uri_copy(xml_base) : NULL;
+    element=raptor_new_xml_element(qname, xml_language, base_uri_copy);
+    if(!element) {
+      raptor_free_qname(qname);
+      if(base_uri_copy)
+        raptor_free_uri(base_uri_copy);
+    }
+  }
+  return element;
+}
+
+
 /**
  * raptor_free_xml_element:
  * @element: XML Element
@@ -101,8 +140,8 @@ raptor_free_xml_element(raptor_xml_element *element)
   if(element->attributes)
     RAPTOR_FREE(raptor_qname_array, element->attributes);
 
-  if(element->content_cdata_length)
-    RAPTOR_FREE(raptor_qname_array, element->content_cdata);
+  if(element->content_cdata_sb)
+    raptor_free_stringbuffer(element->content_cdata_sb);
 
   if(element->base_uri)
     raptor_free_uri(element->base_uri);
@@ -549,7 +588,7 @@ raptor_xml_any_escape_string(const unsigned char *string, size_t len,
   const unsigned char *p;
   unsigned char *q;
   int unichar_len;
-  unsigned long unichar;
+  raptor_unichar unichar;
 
   if(quote != '\"' && quote != '\'')
     quote='\0';
@@ -737,7 +776,7 @@ raptor_iostream_write_xml_any_escaped_string(raptor_iostream* iostr,
 
   for(l=len, p=string; l; p++, l--) {
     int unichar_len=1;
-    unsigned long unichar=*p;
+    raptor_unichar unichar=*p;
 
     if(*p > 0x7f) {
       unichar_len=raptor_utf8_to_unicode_char(&unichar, p, l);
@@ -846,7 +885,7 @@ raptor_xml_name_check(const unsigned char *string, size_t length,
     return 0;
 
   for(pos=0; length > 0; pos++) {
-    unsigned long unichar=0;
+    raptor_unichar unichar=0;
 
     int unichar_len=raptor_utf8_to_unicode_char(&unichar, string, length);
     if(unichar_len < 0 || unichar_len > (int)length)
