@@ -2,7 +2,7 @@
  *
  * rasqal_graph_pattern.c - Rasqal graph pattern class
  *
- * $Id: rasqal_graph_pattern.c 12082 2007-04-22 05:17:12Z dajobe $
+ * $Id: rasqal_graph_pattern.c 13155 2007-11-06 09:38:36Z laalto $
  *
  * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -136,8 +136,10 @@ rasqal_new_graph_pattern_from_sequence(rasqal_query* query,
   rasqal_graph_pattern* gp;
 
   gp=rasqal_new_graph_pattern(query, op);
-  if(!gp)
+  if(!gp) {
+    raptor_free_sequence(graph_patterns);
     return NULL;
+  }
   
   gp->graph_patterns=graph_patterns;
   return gp;
@@ -186,7 +188,7 @@ rasqal_graph_pattern_adjust(rasqal_graph_pattern* gp, int offset)
 /**
  * rasqal_graph_pattern_add_constraint:
  * @gp: #rasqal_graph_pattern query object
- * @expr: #rasqal_expression expr
+ * @expr: #rasqal_expression expr - ownership taken
  *
  * Add a constraint expression to the graph_pattern.
  *
@@ -196,10 +198,15 @@ int
 rasqal_graph_pattern_add_constraint(rasqal_graph_pattern* gp,
                                     rasqal_expression* expr)
 {
-  if(!gp->constraints)
+  if(!gp->constraints) {
     gp->constraints=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression, (raptor_sequence_print_handler*)rasqal_expression_print);
-  raptor_sequence_push(gp->constraints, (void*)expr);
-
+    if(!gp->constraints) {
+      rasqal_free_expression(expr);
+      return 1;
+    }
+  }
+  if(raptor_sequence_push(gp->constraints, (void*)expr))
+    return 1;
   return 0;
 }
 
@@ -256,7 +263,7 @@ rasqal_graph_pattern_get_operator(rasqal_graph_pattern* graph_pattern)
 }
 
 
-static const char* rasqal_graph_pattern_operator_labels[RASQAL_GRAPH_PATTERN_OPERATOR_LAST+1]={
+static const char* const rasqal_graph_pattern_operator_labels[RASQAL_GRAPH_PATTERN_OPERATOR_LAST+1]={
   "UNKNOWN",
   "basic",
   "optional",
@@ -397,3 +404,140 @@ rasqal_graph_pattern_get_index(rasqal_graph_pattern* gp)
   return gp->gp_index;
 }
 
+
+/**
+ * rasqal_graph_pattern_add_sub_graph_pattern:
+ * @graph_pattern: graph pattern to add to
+ * @sub_graph_pattern: graph pattern to add inside
+ *
+ * Add a sub graph pattern to a graph pattern .
+ *
+ **/
+void
+rasqal_graph_pattern_add_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
+                                           rasqal_graph_pattern* sub_graph_pattern)
+{
+  if(!graph_pattern->graph_patterns) {
+    graph_pattern->graph_patterns=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+    if(!graph_pattern->graph_patterns)
+      RASQAL_FATAL1("Out of memory\n");
+  }
+  raptor_sequence_push(graph_pattern->graph_patterns, sub_graph_pattern);
+}
+
+
+/**
+ * rasqal_graph_pattern_get_triple:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ * @idx: index into the sequence of triples in the graph pattern
+ *
+ * Get a triple inside a graph pattern.
+ * 
+ * Return value: #rasqal_triple or NULL if out of range
+ **/
+rasqal_triple*
+rasqal_graph_pattern_get_triple(rasqal_graph_pattern* graph_pattern, int idx)
+{
+  if(!graph_pattern->triples)
+    return NULL;
+
+  idx += graph_pattern->start_column;
+
+  if(idx > graph_pattern->end_column)
+    return NULL;
+  
+  return (rasqal_triple*)raptor_sequence_get_at(graph_pattern->triples, idx);
+}
+
+
+/**
+ * rasqal_graph_pattern_get_sub_graph_pattern_sequence:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ *
+ * Get the sequence of graph patterns inside a graph pattern .
+ * 
+ * Return value:  a #raptor_sequence of #rasqal_graph_pattern pointers.
+ **/
+raptor_sequence*
+rasqal_graph_pattern_get_sub_graph_pattern_sequence(rasqal_graph_pattern* graph_pattern)
+{
+  return graph_pattern->graph_patterns;
+}
+
+
+/**
+ * rasqal_graph_pattern_get_sub_graph_pattern:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ * @idx: index into the sequence of sub graph_patterns in the graph pattern
+ *
+ * Get a sub-graph pattern inside a graph pattern.
+ * 
+ * Return value: #rasqal_graph_pattern or NULL if out of range
+ **/
+rasqal_graph_pattern*
+rasqal_graph_pattern_get_sub_graph_pattern(rasqal_graph_pattern* graph_pattern, int idx)
+{
+  if(!graph_pattern->graph_patterns)
+    return NULL;
+
+  return (rasqal_graph_pattern*)raptor_sequence_get_at(graph_pattern->graph_patterns, idx);
+}
+
+
+/**
+ * rasqal_graph_pattern_get_flags:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ *
+ * Get the graph pattern flags .
+ * 
+ * @deprecated: Always returns 0
+ *
+ * Return value: 0
+ **/
+int
+rasqal_graph_pattern_get_flags(rasqal_graph_pattern* graph_pattern)
+{
+  RASQAL_DEPRECATED_MESSAGE("use rasqal_graph_pattern_get_operator");
+  return 0;
+}
+
+
+/*
+ * rasqal_graph_pattern_set_origin:
+ * @graph_pattern: #rasqal_graph_pattern graph pattern object
+ * @origin: #rasqal_literal variable or URI
+ *
+ * INTERNAL - Set the graph pattern triple origin.
+ * 
+ * All triples in this graph pattern or contained graph patterns are set
+ * to have the given origin.
+ **/
+void
+rasqal_graph_pattern_set_origin(rasqal_graph_pattern* graph_pattern,
+                                rasqal_literal *origin)
+{
+  raptor_sequence* s;
+  
+  s=graph_pattern->triples;
+  if(s) {
+    int i;
+
+    /* Flag all the triples in this graph pattern with origin */
+    for(i= graph_pattern->start_column; i <= graph_pattern->end_column; i++) {
+      rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(s, i);
+      rasqal_triple_set_origin(t, rasqal_new_literal_from_literal(origin));
+    }
+  }
+
+  s=graph_pattern->graph_patterns;
+  if(s) {
+    int i;
+
+    /* Flag all the triples in sub-graph patterns with origin */
+    for(i=0; i < raptor_sequence_size(s); i++) {
+      rasqal_graph_pattern *gp=(rasqal_graph_pattern*)raptor_sequence_get_at(s, i);
+      rasqal_graph_pattern_set_origin(gp, origin);
+    }
+  }
+
+}
