@@ -2,7 +2,7 @@
  *
  * rasqal_query.c - Rasqal RDF Query
  *
- * $Id: rasqal_query.c 12098 2007-04-22 07:31:28Z dajobe $
+ * $Id: rasqal_query.c 13199 2007-11-14 16:33:53Z dajobe $
  *
  * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -82,52 +82,57 @@ rasqal_new_query(const char *name, const unsigned char *uri)
   if(!query)
     return NULL;
   
-  query->context=(char*)RASQAL_CALLOC(rasqal_query_context, 1,
-                                      factory->context_length);
-  if(!query->context) {
-    rasqal_free_query(query);
-    return NULL;
-  }
-  
+  /* set usage first to 1 so we can clean up with rasqal_free_query() on error */
+  query->usage=1;
+
   query->factory=factory;
 
-  query->failed=0;
+  query->limit= -1;
+  query->offset= -1;
 
+  query->genid_counter=1;
+
+  query->context=(char*)RASQAL_CALLOC(rasqal_query_context, 1,
+                                      factory->context_length);
+  if(!query->context)
+    goto tidy;
+  
   raptor_uri_get_handler(&uri_handler, &uri_context);
   query->namespaces=raptor_new_namespaces(uri_handler, uri_context,
                                           (raptor_simple_message_handler)rasqal_query_simple_error,
                                           query,
                                           0);
+  if(!query->namespaces)
+    goto tidy;
 
   query->variables_sequence=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable, (raptor_sequence_print_handler*)rasqal_variable_print);
+  if(!query->variables_sequence)
+    goto tidy;
 
   query->anon_variables_sequence=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable, (raptor_sequence_print_handler*)rasqal_variable_print);
+  if(!query->anon_variables_sequence)
+    goto tidy;
 
   query->triples=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_triple, (raptor_sequence_print_handler*)rasqal_triple_print);
+  if(!query->triples)
+    goto tidy;
   
   query->prefixes=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
-
-  query->query_graph_pattern=NULL;
+  if(!query->prefixes)
+    goto tidy;
 
   query->data_graphs=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_data_graph, (raptor_sequence_print_handler*)rasqal_data_graph_print);
+  if(!query->data_graphs)
+    goto tidy;
 
-  query->distinct= 0;
-  query->limit= -1;
-  query->offset= -1;
-
-  query->order_conditions_sequence=NULL;
-  query->group_conditions_sequence=NULL;
-
-  query->usage=1;
-  
-  query->genid_counter=1;
-  
-  if(factory->init(query, name)) {
-    rasqal_free_query(query);
-    return NULL;
-  }
+  if(factory->init(query, name))
+    goto tidy;
   
   return query;
+
+  tidy:
+  rasqal_free_query(query);
+  return NULL;
 }
 
 
@@ -178,9 +183,6 @@ rasqal_free_query(rasqal_query* query)
   if(query->variable_names)
     RASQAL_FREE(cstrings, query->variable_names);
   
-  if(query->binding_values)
-    RASQAL_FREE(cstrings, query->binding_values);
-  
   if(query->variables)
     RASQAL_FREE(vararray, query->variables);
 
@@ -576,9 +578,10 @@ rasqal_query_add_data_graph(rasqal_query* query,
     return 1;
   
   dg=rasqal_new_data_graph(uri, name_uri, flags);
-  
-  raptor_sequence_push(query->data_graphs, (void*)dg);
-
+  if(!dg)
+    return 1;
+  if(raptor_sequence_push(query->data_graphs, (void*)dg))
+    return 1;
   return 0;
 }
 
@@ -630,8 +633,11 @@ rasqal_query_get_data_graph(rasqal_query* query, int idx)
 void
 rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
 {
-  if(!query->selects)
+  if(!query->selects) {
     query->selects=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+    if(!query->selects)
+      RASQAL_FATAL1("Out of memory\n");
+  }
 
   raptor_sequence_push(query->selects, (void*)var);
 }
@@ -859,9 +865,11 @@ rasqal_query_declare_prefixes(rasqal_query *rq)
 void
 rasqal_query_add_prefix(rasqal_query* query, rasqal_prefix* prefix)
 {
-  if(!query->prefixes)
+  if(!query->prefixes) {
     query->prefixes=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
-  else {
+    if(!query->prefixes)
+      RASQAL_FATAL1("Out of memory\n");
+  } else {
     int i;
     for(i=0; i< raptor_sequence_size(query->prefixes); i++) {
       rasqal_prefix* p=(rasqal_prefix*)raptor_sequence_get_at(query->prefixes, i);
@@ -990,140 +998,6 @@ rasqal_query_get_construct_triple(rasqal_query* query, int idx)
 }
 
 
-/**
- * rasqal_graph_pattern_add_sub_graph_pattern:
- * @graph_pattern: graph pattern to add to
- * @sub_graph_pattern: graph pattern to add inside
- *
- * Add a sub graph pattern to a graph pattern .
- *
- **/
-void
-rasqal_graph_pattern_add_sub_graph_pattern(rasqal_graph_pattern* graph_pattern,
-                                           rasqal_graph_pattern* sub_graph_pattern)
-{
-  if(!graph_pattern->graph_patterns)
-    graph_pattern->graph_patterns=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
-  raptor_sequence_push(graph_pattern->graph_patterns, sub_graph_pattern);
-}
-
-
-/**
- * rasqal_graph_pattern_get_triple:
- * @graph_pattern: #rasqal_graph_pattern graph pattern object
- * @idx: index into the sequence of triples in the graph pattern
- *
- * Get a triple inside a graph pattern.
- * 
- * Return value: #rasqal_triple or NULL if out of range
- **/
-rasqal_triple*
-rasqal_graph_pattern_get_triple(rasqal_graph_pattern* graph_pattern, int idx)
-{
-  if(!graph_pattern->triples)
-    return NULL;
-
-  idx += graph_pattern->start_column;
-
-  if(idx > graph_pattern->end_column)
-    return NULL;
-  
-  return (rasqal_triple*)raptor_sequence_get_at(graph_pattern->triples, idx);
-}
-
-
-/**
- * rasqal_graph_pattern_get_sub_graph_pattern_sequence:
- * @graph_pattern: #rasqal_graph_pattern graph pattern object
- *
- * Get the sequence of graph patterns inside a graph pattern .
- * 
- * Return value:  a #raptor_sequence of #rasqal_graph_pattern pointers.
- **/
-raptor_sequence*
-rasqal_graph_pattern_get_sub_graph_pattern_sequence(rasqal_graph_pattern* graph_pattern)
-{
-  return graph_pattern->graph_patterns;
-}
-
-
-/**
- * rasqal_graph_pattern_get_sub_graph_pattern:
- * @graph_pattern: #rasqal_graph_pattern graph pattern object
- * @idx: index into the sequence of sub graph_patterns in the graph pattern
- *
- * Get a sub-graph pattern inside a graph pattern.
- * 
- * Return value: #rasqal_graph_pattern or NULL if out of range
- **/
-rasqal_graph_pattern*
-rasqal_graph_pattern_get_sub_graph_pattern(rasqal_graph_pattern* graph_pattern, int idx)
-{
-  if(!graph_pattern->graph_patterns)
-    return NULL;
-
-  return (rasqal_graph_pattern*)raptor_sequence_get_at(graph_pattern->graph_patterns, idx);
-}
-
-
-/**
- * rasqal_graph_pattern_get_flags:
- * @graph_pattern: #rasqal_graph_pattern graph pattern object
- *
- * Get the graph pattern flags .
- * 
- * @deprecated: Always returns 0
- *
- * Return value: 0
- **/
-int
-rasqal_graph_pattern_get_flags(rasqal_graph_pattern* graph_pattern)
-{
-  RASQAL_DEPRECATED_MESSAGE("use rasqal_graph_pattern_get_operator");
-  return 0;
-}
-
-
-/*
- * rasqal_graph_pattern_set_origin:
- * @graph_pattern: #rasqal_graph_pattern graph pattern object
- * @origin: #rasqal_literal variable or URI
- *
- * INTERNAL - Set the graph pattern triple origin.
- * 
- * All triples in this graph pattern or contained graph patterns are set
- * to have the given origin.
- **/
-void
-rasqal_graph_pattern_set_origin(rasqal_graph_pattern* graph_pattern,
-                                rasqal_literal *origin)
-{
-  raptor_sequence* s;
-  
-  s=graph_pattern->triples;
-  if(s) {
-    int i;
-
-    /* Flag all the triples in this graph pattern with origin */
-    for(i= graph_pattern->start_column; i <= graph_pattern->end_column; i++) {
-      rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(s, i);
-      rasqal_triple_set_origin(t, rasqal_new_literal_from_literal(origin));
-    }
-  }
-
-  s=graph_pattern->graph_patterns;
-  if(s) {
-    int i;
-
-    /* Flag all the triples in sub-graph patterns with origin */
-    for(i=0; i < raptor_sequence_size(s); i++) {
-      rasqal_graph_pattern *gp=(rasqal_graph_pattern*)raptor_sequence_get_at(s, i);
-      rasqal_graph_pattern_set_origin(gp, origin);
-    }
-  }
-
-}
-
 
 static int
 rasqal_query_prepare_count_graph_patterns(rasqal_query* query,
@@ -1132,9 +1006,11 @@ rasqal_query_prepare_count_graph_patterns(rasqal_query* query,
 {
   raptor_sequence* seq=(raptor_sequence*)data;
 
+  if(raptor_sequence_push(seq, gp)) {
+    rasqal_query_fatal_error(query, "Out of memory in rasqal_query_prepare_count_graph_patterns()");
+    return 1;
+  }
   gp->gp_index=(query->graph_pattern_count++);
-  raptor_sequence_push(seq, gp);
-  
   return 0;
 }
 
@@ -1172,8 +1048,29 @@ rasqal_query_prepare(rasqal_query* query,
   query->prepared=1;
 
   if(query_string) {
-    query->query_string=(unsigned char*)RASQAL_MALLOC(cstring, strlen((const char*)query_string)+1);
-    strcpy((char*)query->query_string, (const char*)query_string);
+    /* flex lexers require two NULs at the end of the lexed buffer.
+     * Add them here instead of parser to allow resource cleanup on error.
+     *  
+     * flex manual:
+     *
+     * Function: YY_BUFFER_STATE yy_scan_buffer (char *base, yy_size_t size)
+     * which scans in place the buffer starting at `base', consisting of
+     * `size' bytes, the last two bytes of which _must_ be
+     * `YY_END_OF_BUFFER_CHAR' (ASCII NUL).  These last two bytes are not
+     * scanned; thus, scanning consists of `base[0]' through
+     * `base[size-2]', inclusive.
+     */
+    int len=strlen((const char*)query_string)+3; /* +3 for " \0\0" */
+    unsigned char *query_string_copy=(unsigned char*)RASQAL_MALLOC(cstring, len);
+    if(!query_string_copy) {
+      query->failed=1;
+      return 1;
+    }
+    strcpy((char*)query_string_copy, (const char*)query_string);
+    query_string_copy[len-3]=' ';
+    query_string_copy[len-2]=query_string_copy[len-1]='\0';
+    query->query_string=query_string_copy;
+    query->query_string_length=len;
   }
 
   if(base_uri)
@@ -1183,16 +1080,15 @@ rasqal_query_prepare(rasqal_query* query,
     base_uri=raptor_new_uri(uri_string);
     raptor_free_memory(uri_string);
   }
-  
-  query->base_uri=base_uri;
-  query->locator.uri=base_uri;
+
+  rasqal_query_set_base_uri(query, base_uri);
   query->locator.line = query->locator.column = query->locator.byte = -1;
 
   rc=query->factory->prepare(query);
   if(rc)
     query->failed=1;
 
-  if(query->query_graph_pattern) {
+  else if(query->query_graph_pattern) {
 #ifndef RASQAL_NO_GP_MERGE
     int modified;
     
@@ -1235,18 +1131,23 @@ rasqal_query_prepare(rasqal_query* query,
       fputs("\n", stdout);
 #endif
 
-    } while(modified);
-#endif
+    } while(modified>0);
+
+    rc=modified; /* error if modified<0, success if modified==0 */
+
+#endif /* !RASQAL_NO_GP_MERGE */
 
     /* Label all graph patterns with an index 0.. for use in discovering
      * the size of the graph pattern execution data array
      */
     query->graph_pattern_count=0;
 
-    /* This sequence stoees shared pointers to the grpah patterns it
+    /* This sequence stores shared pointers to the graph patterns it
      * finds, indexed by the gp_index
      */
     query->graph_patterns_sequence=raptor_new_sequence(NULL, NULL);
+    if(!query->graph_patterns_sequence)
+      return 1;
 
     rasqal_query_graph_pattern_visit(query, 
                                      rasqal_query_prepare_count_graph_patterns,
@@ -1281,6 +1182,9 @@ rasqal_query_execute(rasqal_query* query)
   if(!query_results)
     return NULL;
   
+  /* set executed flag early to enable cleanup on error */
+  query_results->executed=1;
+
   rasqal_query_add_query_result(query, query_results);
 
   rc=rasqal_engine_execute_init(query_results);
@@ -1297,9 +1201,8 @@ rasqal_query_execute(rasqal_query* query)
     }
   }
 
-  query_results->executed=1;
-
-  if(rasqal_engine_execute_run(query_results)) {
+  rc=rasqal_engine_execute_run(query_results);
+  if(rc < 0) {
     rasqal_free_query_results(query_results);
     query_results=NULL;
   }
@@ -1308,7 +1211,7 @@ rasqal_query_execute(rasqal_query* query)
 }
 
 
-static const char* rasqal_query_verb_labels[RASQAL_QUERY_VERB_LAST+1]={
+static const char* const rasqal_query_verb_labels[RASQAL_QUERY_VERB_LAST+1]={
   "Unknown",
   "SELECT",
   "CONSTRUCT",
@@ -1731,7 +1634,8 @@ rasqal_query_write_sparql_literal(sparql_writer_context *wc,
       raptor_iostream_write_byte(iostr, '"');
       raptor_iostream_write_string_ntriples(iostr, l->string, l->string_len, '"');
       raptor_iostream_write_counted_string(iostr, "\"^^", 3);
-      rasqal_query_write_sparql_uri(wc, iostr, rasqal_xsd_datetime_uri);
+      rasqal_query_write_sparql_uri(wc, iostr,
+                                    rasqal_xsd_datatype_type_to_uri(l->type));
       break;
 
     case RASQAL_LITERAL_UNKNOWN:
@@ -1773,7 +1677,7 @@ rasqal_query_write_indent(raptor_iostream* iostr, int indent)
 
   
 
-static const char* rasqal_sparql_op_labels[RASQAL_EXPR_LAST+1]={
+static const char* const rasqal_sparql_op_labels[RASQAL_EXPR_LAST+1]={
   NULL, /* UNKNOWN */
   "&&",
   "||",
@@ -1811,7 +1715,9 @@ static const char* rasqal_sparql_op_labels[RASQAL_EXPR_LAST+1]={
   "REGEX",
   "ASC",   /* GROUP BY ASC */
   "DESC",  /* GROUP BY DESC */
-  "COUNT"
+  "COUNT",
+  NULL, /* VARSTAR */
+  "sameTerm"
 };
 
 
@@ -1879,6 +1785,7 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_GROUP_COND_ASC:
     case RASQAL_EXPR_GROUP_COND_DESC:
     case RASQAL_EXPR_COUNT:
+    case RASQAL_EXPR_SAMETERM:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
       raptor_iostream_write_counted_string(iostr, "( ", 2);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
@@ -2309,10 +2216,10 @@ rasqal_query_escape_counted_string(rasqal_query* query,
                                    size_t* output_len_p)
 {
   raptor_iostream* iostr;
-  unsigned char* output_string=NULL;
+  void* output_string=NULL;
   int rc;
   
-  iostr=raptor_new_iostream_to_string((void**)&output_string, output_len_p,
+  iostr=raptor_new_iostream_to_string(&output_string, output_len_p,
                                       rasqal_alloc_memory);
   if(!iostr)
     return NULL;
@@ -2324,7 +2231,7 @@ rasqal_query_escape_counted_string(rasqal_query* query,
     output_string=NULL;
   }
   
-  return output_string;
+  return (unsigned char *)output_string;
 }
 
 
@@ -2354,3 +2261,11 @@ rasqal_query_get_genid(rasqal_query* query, const unsigned char* base,
 }
 
 
+void
+rasqal_query_set_base_uri(rasqal_query* query, raptor_uri* base_uri)
+{
+  if(query->base_uri)
+    raptor_free_uri(query->base_uri);
+  query->base_uri=base_uri;
+  query->locator.uri=base_uri;
+}
