@@ -201,7 +201,7 @@ extern int rdql_lexer_lex (YYSTYPE *rdql_parser_lval, yyscan_t scanner);
 #define yylex rdql_lexer_lex
 
 
-static int rdql_parse(rasqal_query* rq, const unsigned char *string);
+static int rdql_parse(rasqal_query* rq);
 static int rdql_query_error(rasqal_query* rq, const char *message);
 
 
@@ -1267,47 +1267,47 @@ yydestruct (yymsg, yytype, yyvaluep)
       case 45: /* "\"floating point literal\"" */
 #line 181 "./rdql_parser.y"
 	{ rasqal_free_literal((yyvaluep->literal)); };
-#line 1273 "rdql_parser.c"
+#line 1271 "rdql_parser.c"
 	break;
       case 46: /* "\"string literal\"" */
 #line 181 "./rdql_parser.y"
 	{ rasqal_free_literal((yyvaluep->literal)); };
-#line 1278 "rdql_parser.c"
+#line 1276 "rdql_parser.c"
 	break;
       case 47: /* "\"integer literal\"" */
 #line 181 "./rdql_parser.y"
 	{ rasqal_free_literal((yyvaluep->literal)); };
-#line 1283 "rdql_parser.c"
+#line 1281 "rdql_parser.c"
 	break;
       case 48: /* "\"pattern literal\"" */
 #line 181 "./rdql_parser.y"
 	{ rasqal_free_literal((yyvaluep->literal)); };
-#line 1288 "rdql_parser.c"
+#line 1286 "rdql_parser.c"
 	break;
       case 49: /* "\"boolean literal\"" */
 #line 181 "./rdql_parser.y"
 	{ rasqal_free_literal((yyvaluep->literal)); };
-#line 1293 "rdql_parser.c"
+#line 1291 "rdql_parser.c"
 	break;
       case 50: /* "\"null\"" */
 #line 181 "./rdql_parser.y"
 	{ rasqal_free_literal((yyvaluep->literal)); };
-#line 1298 "rdql_parser.c"
+#line 1296 "rdql_parser.c"
 	break;
       case 51: /* "\"URI literal\"" */
 #line 182 "./rdql_parser.y"
 	{ raptor_free_uri((yyvaluep->uri)); };
-#line 1303 "rdql_parser.c"
+#line 1301 "rdql_parser.c"
 	break;
       case 52: /* "\"QName literal\"" */
 #line 183 "./rdql_parser.y"
 	{ RASQAL_FREE(cstring, (yyvaluep->name)); };
-#line 1308 "rdql_parser.c"
+#line 1306 "rdql_parser.c"
 	break;
       case 53: /* "\"identifier\"" */
 #line 183 "./rdql_parser.y"
 	{ RASQAL_FREE(cstring, (yyvaluep->name)); };
-#line 1313 "rdql_parser.c"
+#line 1311 "rdql_parser.c"
 	break;
 
       default:
@@ -2177,7 +2177,7 @@ yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 2183 "rdql_parser.c"
+#line 2181 "rdql_parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2401,7 +2401,7 @@ yyreturn:
 /* This is declared in rdql_lexer.h but never used, so we always get
  * a warning unless this dummy code is here.  Used once below in an error case.
  */
-static int yy_init_globals (yyscan_t yyscanner ) { return 0; };
+static int yy_init_globals (yyscan_t yyscanner ) { return 0; }
 
 
 
@@ -2428,6 +2428,8 @@ rasqal_rdql_query_engine_init(rasqal_query* rdf_query, const char *name) {
                                          (const unsigned char*)"owl",
                                          (const unsigned char*)RAPTOR_OWL_URI, 0);
 
+  rdf_query->compare_flags = RASQAL_COMPARE_URI;
+
   return 0;
 }
 
@@ -2461,7 +2463,7 @@ rasqal_rdql_query_engine_prepare(rasqal_query* rdf_query) {
   /* for RDQL only, before the graph pattern is made */
   rdf_query->constraints_sequence=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_expression_print);
   
-  rc=rdql_parse(rdf_query, rdf_query->query_string);
+  rc=rdql_parse(rdf_query);
   if(rc)
     return rc;
 
@@ -2500,14 +2502,12 @@ rasqal_rdql_query_engine_execute(rasqal_query* rdf_query,
 
 
 static int
-rdql_parse(rasqal_query* rq, const unsigned char *string) {
+rdql_parse(rasqal_query* rq) {
   rasqal_rdql_query_engine* rqe=(rasqal_rdql_query_engine*)rq->context;
   raptor_locator *locator=&rq->locator;
-  char *buf=NULL;
-  size_t len;
   void *buffer;
   
-  if(!string || !*string)
+  if(!rq->query_string)
     return yy_init_globals(NULL); /* 0 but a way to use yy_init_globals */
 
   locator->line=1;
@@ -2525,28 +2525,10 @@ rdql_parse(rasqal_query* rq, const unsigned char *string) {
 
   rdql_lexer_set_extra(((rasqal_query*)rq), rqe->scanner);
 
-  /* This
-   *   buffer= rdql_lexer__scan_string((const char*)string, rqe->scanner);
-   * is replaced by the code below.  
-   * 
-   * The extra space appended to the buffer is the least-pain
-   * workaround to the lexer crashing by reading EOF twice in
-   * rdql_copy_regex_token; at least as far as I can diagnose.  The
-   * fix here costs little to add as the above function does
-   * something very similar to this anyway.
-   */
-  len= strlen((const char*)string);
-  buf= (char *)RASQAL_MALLOC(cstring, len+3);
-  strncpy(buf, (const char*)string, len);
-  buf[len]= ' ';
-  buf[len+1]= buf[len+2]='\0'; /* YY_END_OF_BUFFER_CHAR; */
-  buffer= rdql_lexer__scan_buffer(buf, len+3, rqe->scanner);
+  buffer= rdql_lexer__scan_buffer((char*)rq->query_string, rq->query_string_length, rqe->scanner);
 
   rdql_parser_parse(rq);
 
-  if(buf)
-    RASQAL_FREE(cstring, buf);
-
   rdql_lexer_lex_destroy(rqe->scanner);
   rqe->scanner_set=0;
 
