@@ -2,7 +2,7 @@
  *
  * rdfproc.c - Redland RDF command processor
  *
- * $Id: rdfproc.c 12153 2007-04-24 06:24:09Z dajobe $
+ * $Id: rdfproc.c 12987 2007-10-22 07:42:51Z laalto $
  *
  * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -125,7 +125,7 @@ static command commands[]={
   {CMD_CONTEXTS, "contexts", 0, 0, 0},
   {CMD_MATCH, "match", 3, 4, 0},
   {CMD_SIZE, "size", 0, 0, 0},
-  {(enum command_type)-1, NULL}  
+  {(enum command_type)-1, NULL, 0, 0, 0}  
 };
  
 
